@@ -7,15 +7,15 @@ open Printf
 
 let counter = ref 0
 
-let hit_counter env cgi =
+let hit_counter env (cgi:Netcgi.cgi) =
   let cur_counter = !counter in
   incr counter;
   cgi # output # output_string "<html><body>\n";
   cgi # output # output_string (sprintf "Counter = %d<br>\n" cur_counter);
   List.iter
-    (fun (name, arg) ->
+    (fun arg ->
        cgi # output # output_string (sprintf "Arg %s = %s<br>\n"
-				   (Netencoding.Html.encode_from_latin1 name)
+				   (Netencoding.Html.encode_from_latin1 arg#name)
 				   (Netencoding.Html.encode_from_latin1 arg#value))
     )
     cgi # arguments;
@@ -66,23 +66,7 @@ let srv =
 
 
 let start () =
-  let config : http_reactor_config =
-    object
-      method config_timeout_next_request = 15.0
-      method config_timeout = 300.0
-      method config_reactor_synch = `Write
-      method config_cgi = Netcgi_env.default_config
-      method config_error_response n = "<html>Error " ^ string_of_int n ^ "</html>"
-      method config_log_error _ _ _ _ msg =
-        printf "Error log: %s\n" msg
-      method config_max_reqline_length = 256
-      method config_max_header_length = 32768
-      method config_max_trailer_length = 32768
-      method config_limit_pipeline_length = 5
-      method config_limit_pipeline_size = 250000
-      method config_announce_server = `Ocamlnet
-    end in
-
+  let config = Nethttpd_reactor.default_http_reactor_config in
   let master_sock = Unix.socket Unix.PF_INET Unix.SOCK_STREAM 0 in
   Unix.setsockopt master_sock Unix.SO_REUSEADDR true;
   Unix.bind master_sock (Unix.ADDR_INET(Unix.inet_addr_any, 8765));
@@ -100,5 +84,30 @@ let start () =
   done
 ;;
 
-Sys.set_signal Sys.sigpipe Sys.Signal_ignore;
+let conf_debug() =
+  (* Set the environment variable DEBUG to either:
+       - a list of Netlog module names
+       - the keyword "ALL" to output all messages
+       - the keyword "LIST" to output a list of modules
+     By setting DEBUG_WIN32 additional debugging for Win32 is enabled.
+   *)
+  let debug = try Sys.getenv "DEBUG" with Not_found -> "" in
+  if debug = "ALL" then
+    Netlog.Debug.enable_all()
+  else if debug = "LIST" then (
+    List.iter print_endline (Netlog.Debug.names());
+    exit 0
+  )
+  else (
+    let l = Netstring_str.split (Netstring_str.regexp "[ \t\r\n]+") debug in
+    List.iter
+      (fun m -> Netlog.Debug.enable_module m)
+      l
+  );
+  if (try ignore(Sys.getenv "DEBUG_WIN32"); true with Not_found -> false) then
+    Netsys_win32.Debug.debug_c_wrapper true
+;;
+
+Netsys_signal.init();
+conf_debug();
 start();;
