@@ -1,9 +1,9 @@
-(* $Id: qclient.ml 286 2006-04-29 16:21:42Z gerd $
+(* $Id: qclient.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
 
-open Rtypes
+open Netnumber
 open Rpc
 open Rpc_client
 open Printf
@@ -767,6 +767,10 @@ let start() =
 
 
 let main() =
+  Printexc.record_backtrace true;
+  Rpc_client.Debug.enable := true;
+  Rpc_client.Debug.enable_ptrace := true;
+  Rpc_client.Debug.ptrace_verbosity := `Name_full_args; 
   try
     start();
     exit 0
