@@ -1,4 +1,4 @@
-(* $Id: unixqueue_pollset.ml 1624 2011-06-13 12:27:58Z gerd $ *)
+(* $Id: unixqueue_pollset.ml 1710 2012-02-16 15:15:27Z gerd $ *)
 
 (* pset # dispose: we try to call it when there are no more operations in
    the queue. This is tested when
@@ -6,6 +6,8 @@
     - the group is cleared (and we are not waiting)
     - after every wait
    Note that we must not call [dispose] while [wait] is running!
+
+   Also, we call it when [run] is left by an exception.
  *)
 
 
@@ -1011,6 +1013,7 @@ object(self)
       done;
     with
       | error ->
+	  pset # dispose();
           raise error
 
   method is_running =
