@@ -1,5 +1,5 @@
 (* 
- * $Id: uq_socks5.ml 1662 2011-08-29 23:05:06Z gerd $
+ * $Id: uq_socks5.ml 2221 2015-02-23 22:19:42Z gerd $
  *)
 
 open Uq_engines;;
@@ -14,6 +14,7 @@ let addr_of_name name =
   entry.Unix.h_addr_list.(0)
 ;;
 
+(*
 class [ 't ] engine_mixin (init_state : 't engine_state) =
 object(self)
   val mutable notify_list = []
@@ -36,7 +37,7 @@ object(self)
     notify_list_new <- [];
     notify_list <- List.filter (fun f -> f()) notify_list
 end ;;
-
+ *)
 
 class message_receiver src message_callback ues =
 object(self)
@@ -52,7 +53,7 @@ object(self)
    * The engine is never done.
    *)
 
-  inherit [unit] engine_mixin (`Working 0 : unit engine_state)
+  inherit [unit] engine_mixin (`Working 0 : unit engine_state) ues
 
   val group = Unixqueue.new_group ues
   val mutable in_eof = false
@@ -86,7 +87,7 @@ object(self)
     self # check_input_polling()
 
   method abort() =
-    match state with
+    match self#state with
 	`Working _ ->
 	  in_eof <- true;
 	  self # set_state `Aborted;
@@ -97,7 +98,7 @@ object(self)
   method event_system = ues
 
   method private count() =
-    match state with
+    match self#state with
 	`Working n -> 
 	  self # set_state (`Working (n+1))
       | _ ->
@@ -152,7 +153,7 @@ object(self)
     (* It is possible that this method is called when the engine is already
      * aborted. In this case, just do nothing.
      *)
-    match state with
+    match self#state with
 	`Working _ ->
 	  let message_expected = String.length message in
 	  let need_polling = not in_eof && message_length < message_expected in
@@ -411,7 +412,7 @@ type 't transition =
 
 class virtual ['t] automaton socks_socket ues =
 object(self)
-  inherit ['t] engine_mixin (`Working 0)
+  inherit ['t] engine_mixin (`Working 0) ues
 
   val mutable socks_state = Trans(fun () -> assert false)
 
@@ -419,7 +420,7 @@ object(self)
   val mutable socks_in = lazy (assert false)
 
   initializer
-    socks_out <- lazy(new output_async_descr 
+    socks_out <- lazy(new Uq_transfer.output_async_descr 
 			~close_dst:false ~dst:socks_socket ues);
     socks_in <- lazy(new message_receiver 
 		       socks_socket self#msg_received ues);
@@ -444,7 +445,7 @@ object(self)
 
   method private msg_received () =
     try
-      ( match state with
+      ( match self#state with
 	    `Working n ->
 	      self # set_state (`Working (n+1))
 	  | _ ->
@@ -684,11 +685,11 @@ object(self)
     (* First connect to the SOCKS proxy; if connected, invoke
      * the SOCKS state automaton
      *)
-    let proxy_conn_eng = connector ?proxy socks_connaddr ues in
+    let proxy_conn_eng = Uq_client.connect_e ?proxy socks_connaddr ues in
     new seq_engine
       proxy_conn_eng
       (fun conn_stat ->
-	 let socks_socket = client_endpoint conn_stat in
+	 let socks_socket = Uq_client.client_endpoint conn_stat in
 	 let udp_socket = Unix.socket Unix.PF_INET Unix.SOCK_DGRAM 0 in
 	 Unix.set_nonblock udp_socket;
 	 Netsys.set_close_on_exec udp_socket;
@@ -737,7 +738,7 @@ end
 ;;
 
 
-class socks5_socket_connector ?proxy socks_connaddr : client_socket_connector =
+class socks5_socket_connector ?proxy socks_connaddr : client_endpoint_connector =
 object(self)
 
   method connect connaddr ues =
@@ -756,11 +757,11 @@ object(self)
 	  (* First connect to the SOCKS proxy; if connected, invoke
 	   * the SOCKS state automaton
 	   *)
-	  let proxy_conn_eng = connector ?proxy socks_connaddr ues in
+	  let proxy_conn_eng = Uq_client.connect_e ?proxy socks_connaddr ues in
 	  new seq_engine
 	    proxy_conn_eng
 	    (fun conn_stat ->
-	       let socks_socket = client_endpoint conn_stat in
+	       let socks_socket = Uq_client.client_endpoint conn_stat in
 	       let eng =
 		 new socks5_connect_automaton socks_socket sockspec ues in
 	       when_state
@@ -804,7 +805,7 @@ end
 
 
 class socks5_socket_acceptor socks_socket actual_bind_spec ues
-      : server_socket_acceptor =
+      : server_endpoint_acceptor =
   (* actual_bind_spec: The bind address the SOCKS proxy really uses *)
 object(self)
 
@@ -814,7 +815,7 @@ object(self)
   val mutable responsible_for_socks_socket = true
 
   method server_address = `Socket(actual_bind_spec,
-				  Uq_engines.default_connect_options)
+				  Uq_client.default_connect_options)
 
   method multiple_connections = false
 
@@ -847,7 +848,7 @@ end
 
 class socks5_listen_automaton socks_socket bind_spec ues =
 object(self)
-  inherit [server_socket_acceptor] automaton socks_socket ues
+  inherit [server_endpoint_acceptor] automaton socks_socket ues
 
   initializer
     let socks_in = Lazy.force socks_in in
@@ -876,7 +877,7 @@ end
 ;;
 
 
-class socks5_socket_listener ?proxy socks_connaddr : server_socket_listener =
+class socks5_socket_listener ?proxy socks_connaddr : server_endpoint_listener =
 object(self)
   method listen lstnaddr ues =
     match lstnaddr with
@@ -892,11 +893,11 @@ object(self)
 	  (* First connect to the SOCKS proxy; if connected, invoke
 	   * the SOCKS state automaton
 	   *)
-	  let proxy_conn_eng = connector ?proxy socks_connaddr ues in
+	  let proxy_conn_eng = Uq_client.connect_e ?proxy socks_connaddr ues in
 	  new seq_engine
 	    proxy_conn_eng
 	    (fun conn_stat ->
-	       let socks_socket = client_endpoint conn_stat in
+	       let socks_socket = Uq_client.client_endpoint conn_stat in
 	       let eng = new socks5_listen_automaton socks_socket sockspec ues
 	       in
 	       when_state
