@@ -1,4 +1,4 @@
-(* $Id: nethttpd_kernel.ml 1641 2011-07-20 19:42:58Z gerd $
+(* $Id: nethttpd_kernel.ml 2195 2015-01-01 12:23:39Z gerd $
  *
  *)
 
@@ -44,6 +44,7 @@ type fatal_error =
     | `Message_too_long
     | `Timeout
     | `Unix_error of Unix.error
+    | `TLS_error of string * string
     | `Server_error
     ]
 
@@ -54,6 +55,7 @@ let string_of_fatal_error =
     | `Message_too_long -> "Nethttpd: Message too long, dropping it"
     | `Timeout -> "Nethttpd: Connection timed out"
     | `Unix_error ue -> ("Nethttpd: System error: " ^ Unix.error_message ue)
+    | `TLS_error(code,msg) -> ("Nethttpd: TLS error code " ^ code ^ ": " ^ msg)
     | `Server_error -> "Nethttpd: Terminating connection because of internal error"
 	
 
@@ -236,7 +238,7 @@ object(self)
 		  let b = Netbuffer.create 256 in 
 		     (* Expect a short/empty header in most cases *)
 		  let ch = new Netchannels.output_netbuffer b in
-		  Mimestring.write_header ch info_header#fields;
+		  Netmime_string.write_header ch info_header#fields;
 		  Queue.push 
 		    (`Resp_wire_data (Netbuffer.unsafe_buffer b, 0, Netbuffer.length b))
 		    queue;
@@ -324,7 +326,7 @@ object(self)
 	  (* Convert the header to a data chunk: *)
 	  let b = Netbuffer.create 4096 in
 	  let ch = new Netchannels.output_netbuffer b in
-	  Mimestring.write_header ch resp_header#fields;
+	  Netmime_string.write_header ch resp_header#fields;
 	  Queue.push 
 	    (`Resp_wire_data (Netbuffer.unsafe_buffer b, 0, Netbuffer.length b)) queue;
 	  (* What is accepted next: *)
@@ -594,16 +596,26 @@ object
   method config_limit_pipeline_size : int
   method config_announce_server : announcement
   method config_suppress_broken_pipe : bool
+  method config_tls : Netsys_crypto_types.tls_config option
+end
+
+
+class type http_protocol_hooks =
+object
+  method tls_set_cache : store:(string -> string -> unit) ->
+                         remove:(string -> unit) ->
+                         retrieve:(string -> string) ->
+                            unit
 end
 
 
 let http_find_line_start s pos len =
-  try Mimestring.find_line_start s pos len
+  try Netmime_string.find_line_start s pos len
   with Not_found -> raise Buffer_exceeded
 
 
 let http_find_double_line_start s pos len =
-  try Mimestring.find_double_line_start s pos len
+  try Netmime_string.find_double_line_start s pos len
   with Not_found -> raise Buffer_exceeded
 
 
@@ -675,7 +687,7 @@ let parse_chunk_header s pos len =
   let c1 = s.[p1] in
   if c1=';' then (
     let p2 =
-      Mimestring.find_line_start s p1 (e - p1) in
+      Netmime_string.find_line_start s p1 (e - p1) in
     if p2 <> e then raise Not_found
   )
   else (
@@ -698,7 +710,52 @@ module StrSet = Set.Make(String)
 class http_protocol (config : #http_protocol_config) (fd : Unix.file_descr) =
   let pa = Netsys_posix.create_poll_array 1 in
   let fdi = Netsys.int64_of_file_descr fd in
+  let tls =
+    match config # config_tls with
+      | None -> None
+      | Some tc -> 
+          Some(Netsys_tls.endpoint
+                 (Netsys_tls.create_file_endpoint 
+                    ~role:`Server ~rd:fd ~wr:fd ~peer_name:None tc))
+  in
+  let tls_message code =
+    match config # config_tls with
+      | None -> code
+      | Some tc ->
+           let module Config = (val tc : Netsys_crypto_types.TLS_CONFIG) in
+           Config.TLS.error_message code in
+
+  let hooks =
+    ( object
+        method tls_set_cache ~store ~remove ~retrieve =
+          match tls with
+            | None ->
+                 ()
+            | Some ep ->
+                 let module Endpoint = 
+                   (val ep : Netsys_crypto_types.TLS_ENDPOINT) in
+                 Endpoint.TLS.set_session_cache
+                   ~store ~remove ~retrieve Endpoint.endpoint
+      end
+    ) in
 object(self)
+  val mutable override_dir = None
+    (* For TLS: can be set to [Some `R] or [Some `W] if the descriptor needs
+       to be read or written
+     *)
+
+  val mutable tls_handshake = (tls <> None)
+    (* Whether the handshake is not yet complete *)
+
+  val mutable tls_shutdown = false
+    (* Whether a TLS shutdown (for sending) needs to be done *)
+
+  val mutable tls_shutdown_done = false
+    (* Whether the TLS shutdown (for sending) is over *)
+
+  val mutable tls_session_props = None
+    (* Session properties (available after handshake) *)
+
   val mutable resp_queue = Queue.create()
     (* The queue of [http_response] objects. The first is currently being transmitted *)
 
@@ -710,6 +767,11 @@ object(self)
      * longer interested in any input because of processing errors
      *)
 
+  val mutable recv_fd_eof = false
+    (* Whether the descriptor is at EOF. This can be different from recv_eof
+       if TLS is active.
+     *) 
+
   val mutable recv_queue = (Queue.create() : (req_token * int) Queue.t)
     (* The queue of received tokens. The integer is the estimated buffer size *)
 
@@ -742,16 +804,26 @@ object(self)
     Unix.set_nonblock fd
   )
 
+  method hooks = hooks
+
   method cycle ?(block=0.0) () = 
+    override_dir <- None;
     try
       (* Block until we have something to read or write *)
       if block <> 0.0 then
 	self # block block;
-      (* Accept any arriving data, and process that *)
-      self # accept_data();
-      (* Transmit any outgoing data *)
-      self # transmit_response();
-
+      (* Maybe the TLS handshake is in progress: *)
+      if tls_handshake then
+        self # do_tls_handshake()
+      else
+        if tls_shutdown && not tls_shutdown_done then
+          self # do_tls_shutdown()
+        else (
+          (* Accept any arriving data, and process that *)
+          self # accept_data();
+          (* Transmit any outgoing data *)
+          self # transmit_response();
+        )
 
     with
       | Fatal_error e ->
@@ -833,6 +905,54 @@ object(self)
   method test_coverage =
     StrSet.elements test_coverage
 
+  (* ---- TLS ---- *)
+
+  method private do_tls_handshake() =
+    try
+      ( match tls with
+          | None -> ()
+          | Some t ->
+              Netsys_tls.handshake t;
+              tls_handshake <- false;
+      )
+    with
+      | Netsys_types.EAGAIN_RD ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake EAGAIN_RD" fdi);
+          override_dir <- Some `R
+      | Netsys_types.EAGAIN_WR ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake EAGAIN_WR" fdi);
+          override_dir <- Some `W
+      | Unix.Unix_error(Unix.EINTR,_,_) ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake EINTR" fdi)
+      | Netsys_types.TLS_error code as e ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake TLS_ERROR %s" fdi
+                                   (Netexn.to_string e));
+          self # abort(`TLS_error(code,tls_message code))
+
+
+  method private do_tls_shutdown() =
+    try
+      ( match tls with
+          | None -> ()
+          | Some t ->
+              Netsys_tls.shutdown t Unix.SHUTDOWN_SEND
+      );
+      tls_shutdown_done <- true
+    with
+      | Netsys_types.EAGAIN_RD ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake EAGAIN_RD" fdi);
+          override_dir <- Some `R
+      | Netsys_types.EAGAIN_WR ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake EAGAIN_WR" fdi);
+          override_dir <- Some `W
+      | Unix.Unix_error(Unix.EINTR,_,_) ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake EINTR" fdi)
+      | Netsys_types.TLS_error code as e ->
+	  dlogr (fun () -> sprintf "FD %Ld: handshake TLS_ERROR %s" fdi
+                                   (Netexn.to_string e));
+          self # abort (`TLS_error(code,tls_message code))
+
+
   (* ---- Process received data ---- *)
 
   method private stop_input_acceptor() =
@@ -862,11 +982,23 @@ object(self)
 	    dlogr (fun () -> sprintf "FD %Ld: recv" fdi);
 	    let n = Netbuffer.add_inplace   (* or Unix_error *)
 	      recv_buf
-	      (fun s pos len -> Unix.recv fd s pos len []) in
+	      (fun s pos len -> 
+                 match tls with
+                   | None -> Unix.recv fd s pos len []
+                   | Some t -> Netsys_tls.recv t s pos len
+              ) in
 	    if n=0 then (
 	      recv_eof <- true;
-	      need_linger <- false;
-	      dlog (sprintf "FD %Ld: got EOF" fdi);
+              ( match tls with
+                  | None -> 
+                      recv_fd_eof <- true;
+                      need_linger <- false
+                  | Some t ->
+                      recv_fd_eof <- Netsys_tls.at_transport_eof t;
+                      tls_shutdown <- true;
+              );
+	      dlog (sprintf "FD %Ld: got EOF (fd_eof=%B)" 
+                            fdi recv_fd_eof);
 	    )
 	    else
 	      dlogr (fun () -> sprintf "FD %Ld: got %d bytes" fdi n);
@@ -876,7 +1008,8 @@ object(self)
 	    false (* no new data *)
 	)
       with
-	| Unix.Unix_error((Unix.EAGAIN | Unix.EWOULDBLOCK), _,_)->
+	| Unix.Unix_error((Unix.EAGAIN | Unix.EWOULDBLOCK), _,_)
+        | Netsys_types.EAGAIN_RD ->
 	    dlogr (fun () -> sprintf "FD %Ld: recv EWOULDBLOCK" fdi);
 	    false             (* socket not ready *)
 	| Unix.Unix_error(Unix.EINTR,_,_) ->
@@ -890,6 +1023,14 @@ object(self)
 	    dlogr (fun () -> sprintf "FD %Ld: recv ERROR" fdi);
 	    self # abort (`Unix_error e);
 	    false
+        | Netsys_types.EAGAIN_WR ->
+            (* Currently impossible *)
+            assert false
+        | Netsys_types.TLS_error code as e ->
+	    dlogr (fun () -> sprintf "FD %Ld: rev TLS_ERROR %s" fdi
+                                     (Netexn.to_string e));
+            self # abort(`TLS_error(code,tls_message code));
+            false
     in
     if continue then 
       self # accept_loop()
@@ -939,15 +1080,19 @@ object(self)
      * If we ever hit the bounding of the buffer, raise Buffer_exceeded. This means
      * we don't have the header block yet.
      *)
-    IFDEF Testing THEN self # case "accept_header" ELSE () END;
+    #ifdef Testing
+       self # case "accept_header";
+    #endif
     waiting_for_next_message <- true;
     let l = Netbuffer.length recv_buf in
     let s = Netbuffer.unsafe_buffer recv_buf in
-    let block_start = Mimestring.skip_line_ends s pos (l - pos) in  (* (1) *)
+    let block_start = Netmime_string.skip_line_ends s pos (l - pos) in  (* (1) *)
     try
       (* (2) *)
       if block_start = l || (block_start+1 = l && s.[block_start] = '\013') then (
-	IFDEF Testing THEN self # case "accept_header/1" ELSE () END;
+	#ifdef Testing
+           self # case "accept_header/1";
+        #endif
 	raise Buffer_exceeded;
       );
       (* (2a) *)
@@ -956,7 +1101,9 @@ object(self)
 	  http_find_line_start s block_start (l - block_start)
 	with
 	    Buffer_exceeded ->
-	      IFDEF Testing THEN self # case "accept_header/reqline_ex" ELSE () END;
+	      #ifdef Testing
+                 self # case "accept_header/reqline_ex";
+              #endif
 	      waiting_for_next_message <- false;
 	      if l-block_start > config#config_max_reqline_length then
 		raise (Bad_request `Request_line_too_long);
@@ -970,7 +1117,9 @@ object(self)
 	  let (meth, uri, proto_s) = 
 	    parse_req_line s block_start (reqline_end - block_start) in
 	    (* or Not_found *)
-	  IFDEF Testing THEN self # case "accept_header/4" ELSE () END;
+	  #ifdef Testing
+             self # case "accept_header/4";
+          #endif
 	  let proto = protocol_of_string proto_s in
 	  ( match proto with
 	      | `Http((1,_),_) -> ()
@@ -980,7 +1129,9 @@ object(self)
 	with
 	  | Not_found ->
 	      (* This is a bad request. Response should be "Bad Request" *)
-	      IFDEF Testing THEN self # case "accept_header/3" ELSE () END;
+	      #ifdef Testing
+                 self # case "accept_header/3";
+              #endif
 	      raise (Bad_request `Bad_request_line)
       in
       (* (3) *)
@@ -990,7 +1141,9 @@ object(self)
 	  http_find_double_line_start s block_start (l - block_start)
 	with
 	    Buffer_exceeded -> 
-	      IFDEF Testing THEN self # case "accept_header/2" ELSE () END;
+	      #ifdef Testing
+                 self # case "accept_header/2";
+              #endif
 	      if l-block_start > config_max_header_length then
 		raise (Fatal_error `Message_too_long);
 	      raise Buffer_exceeded
@@ -1008,10 +1161,33 @@ object(self)
        * in this case where we only read from a constant string.
        *)
       let req_h = 
-	try Netmime.read_mime_header str 
+	try Netmime_channels.read_mime_header str 
 	with Failure _ -> 
-	  IFDEF Testing THEN self # case "accept_header/5" ELSE () END;
+	  #ifdef Testing
+             self # case "accept_header/5";
+          #endif
 	  raise(Bad_request `Bad_header) in
+      (* TLS: check whether Host header (if set) equals the SNI host name *)
+      ( match self#tls_session_props with
+          | None ->
+               ()
+          | Some props ->
+               ( try
+                   let host_hdr =
+                     match req_h # multiple_field "host" with
+                       | [ host_hdr ] -> host_hdr
+                       | [] -> raise Not_found
+                       | _ -> raise(Bad_request (`Bad_header_field "Host")) in
+                   let host_name, _ =
+                     try Nethttp.split_host_port host_hdr
+                     with _ -> host_hdr, None in
+                   if not (Nettls_support.is_endpoint_host host_name props)
+                   then
+                     raise(Bad_request (`Bad_header_field "Host"))
+                 with Not_found ->  (* No "Host" header => no checks *)
+                   ()
+               )
+      );
       (* (5) *)
       let close = 
 	match req_version with
@@ -1031,20 +1207,28 @@ object(self)
 
     with
       | Buffer_exceeded ->
-	  IFDEF Testing THEN self # case "accept_header/exceeded" ELSE () END;
+	  #ifdef Testing
+             self # case "accept_header/exceeded";
+          #endif
 	  if recv_eof then (
 	    if l = block_start then (   (* Regular EOF *)
-	      IFDEF Testing THEN self # case "accept_header/regeof" ELSE () END;
+	      #ifdef Testing
+                self # case "accept_header/regeof";
+              #endif
 	      self # push_recv (`Eof, 0);
 	      `Restart_with (fun () -> `Restart)
 	    )
 	    else (
-	      IFDEF Testing THEN self # case "accept_header/eof" ELSE () END;
+	      #ifdef Testing
+                 self # case "accept_header/eof";
+              #endif
 	      raise(Bad_request `Unexpected_eof)
 	    )
 	  )
 	  else (
-	    IFDEF Testing THEN self # case "accept_header/restart" ELSE () END;
+	    #ifdef Testing
+               self # case "accept_header/restart";
+            #endif
 	    Netbuffer.delete recv_buf 0 block_start;
 	    `Restart_with (self # accept_header 0)
 	  )
@@ -1054,7 +1238,9 @@ object(self)
      * only checks the transfer encoding, and passes over to
      * [accept_body_identity] or [accept_body_chunked].
      *)
-    IFDEF Testing THEN self # case "accept_body_start" ELSE () END;
+    #ifdef Testing
+       self # case "accept_body_start";
+    #endif
     let is_http_1_1 =
       function
 	| `Http((1,1),_) -> true
@@ -1069,7 +1255,9 @@ object(self)
 	      let rfc2616_expect = 
 		List.exists (fun (tok,_,_) -> tok = "100-continue") expect_list in
 	      if rfc2068_expect || rfc2616_expect then (
-		IFDEF Testing THEN self # case "accept_body_start/100-continue" ELSE () END;
+		#ifdef Testing
+                  self # case "accept_body_start/100-continue";
+                #endif
 		self # push_recv (`Req_expect_100_continue, 0);
 		if resp#state = `Inhibited then
 		  resp # set_state `Queued   (* allow response from now on *)
@@ -1083,23 +1271,27 @@ object(self)
 	    [] | ["identity",_] -> false
 	  | _ -> true in
       if chunked_encoding then (
-	IFDEF Testing THEN self # case "accept_body_start/chunked" ELSE () END;
+	#ifdef Testing
+           self # case "accept_body_start/chunked";
+        #endif
 	`Continue (self # accept_body_chunked req_h resp pos)
       )
       else (
 	let remaining_length = 
 	  try Some(get_content_length req_h) with Not_found -> None in
-	IFDEF Testing THEN 
+	#ifdef Testing
 	  if remaining_length = None then
 	    self # case "accept_body_start/empty" 
 	  else
-	    self # case "accept_body_start/identity" 
-	ELSE () END;
+	    self # case "accept_body_start/identity";
+	#endif
 	`Continue (self # accept_body_identity req_h resp pos remaining_length)
       )
     with
 	Bad_header_field name ->
-	  IFDEF Testing THEN self # case "accept_body_start/bad_header_field" ELSE () END;
+	  #ifdef Testing
+             self # case "accept_body_start/bad_header_field";
+          #endif
 	  raise(Bad_request (`Bad_header_field name))
 
   method private accept_body_identity req_h resp pos remaining_length () : cont =
@@ -1108,7 +1300,9 @@ object(self)
      * the body is complete. [None] means there was neither [Content-length] nor
      * [Transfer-Encoding], so the body is empty (e.g. for GET).
      *)
-    IFDEF Testing THEN self # case "accept_body_identity" ELSE () END;
+    #ifdef Testing
+       self # case "accept_body_identity";
+    #endif
     let l = Netbuffer.length recv_buf in
     match remaining_length with
       | Some rl ->
@@ -1119,23 +1313,31 @@ object(self)
 	    self # push_recv (`Req_body(Netbuffer.sub recv_buf pos n, 0, n), n);
 	  let rl' = Int64.sub rl take_length in
 	  if rl' > 0L then (
-	    IFDEF Testing THEN self # case "accept_body_identity/exceeded" ELSE () END;
+	    #ifdef Testing
+               self # case "accept_body_identity/exceeded";
+            #endif
 	    (* We hit the buffer boundary *)
 	    if recv_eof then (
 	      (* This request was prematurely terminated by EOF. Simply drop it. *)
-	      IFDEF Testing THEN self # case "accept_body_identity/eof" ELSE () END;
+	      #ifdef Testing
+                self # case "accept_body_identity/eof";
+              #endif
 	      raise(Bad_request `Unexpected_eof)
 	    )
 	    else (
 	      (* Need to read the remaining part of the request: *)
-	      IFDEF Testing THEN self # case "accept_body_identity/restart" ELSE () END;
+	      #ifdef Testing
+                 self # case "accept_body_identity/restart";
+              #endif
 	      Netbuffer.clear recv_buf;
 	      `Restart_with(self # accept_body_identity req_h resp 0 (Some rl'))
 	    )
 	  )
 	  else (
 	    (* This was the last part of the message. *)
-	    IFDEF Testing THEN self # case "accept_body_identity/last" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_identity/last";
+            #endif
 	    self # push_recv (`Req_end, 0);
 	    pipeline_len <- pipeline_len + 1;
 	    if resp#state = `Inhibited then
@@ -1153,7 +1355,9 @@ object(self)
     (* Check for a chunk header at byte position [pos]. If complete, parse the number of
      * bytes the chunk will consist of, and continue with [accept_body_chunked_contents].
      *)
-    IFDEF Testing THEN self # case "accept_body_chunked" ELSE () END;
+    #ifdef Testing
+       self # case "accept_body_chunked";
+    #endif
     let l = Netbuffer.length recv_buf in
     let s = Netbuffer.unsafe_buffer recv_buf in
     try
@@ -1164,44 +1368,64 @@ object(self)
 	  let chunk_length =
 	    try int_of_string("0x" ^ hex_digits)
 	    with Failure _ -> 
-	      IFDEF Testing THEN self # case "accept_body_chunked/ch_large" ELSE () END;
+	      #ifdef Testing
+                self # case "accept_body_chunked/ch_large";
+              #endif
 	      raise(Bad_request (`Format_error "Chunk too large")) in
 	  (* Continue with chunk data or chunk end *)
 	  if chunk_length > 0 then (
-	    IFDEF Testing THEN self # case "accept_body_chunked/go_on" ELSE () END;
+	    #ifdef Testing
+               self # case "accept_body_chunked/go_on";
+            #endif
 	    `Continue(self # accept_body_chunked_contents req_h resp p chunk_length)
 	  )
 	  else (
-	    IFDEF Testing THEN self # case "accept_body_chunked/end" ELSE () END;
+	    #ifdef Testing
+               self # case "accept_body_chunked/end";
+            #endif
 	    `Continue(self # accept_body_chunked_end req_h resp p)
 	  )
 	with Not_found ->
-	  IFDEF Testing THEN self # case "accept_body_chunked/invalid_ch" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked/invalid_ch";
+          #endif
 	  raise(Bad_request (`Format_error "Invalid chunk"))
       )
     with
       | Buffer_exceeded -> 
-	  IFDEF Testing THEN self # case "accept_body_chunked/exceeded" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked/exceeded";
+          #endif
 	  if recv_eof then (
-	    IFDEF Testing THEN self # case "accept_body_chunked/eof" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_chunked/eof";
+            #endif
 	    raise(Bad_request `Unexpected_eof);
 	  );
 	  if pos > 0 then
 	    Netbuffer.delete recv_buf 0 pos;
 	  if Netbuffer.length recv_buf > 500 then (
-	    IFDEF Testing THEN self # case "accept_body_chunked/ch_hdr_large" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_chunked/ch_hdr_large";
+            #endif
 	    raise(Bad_request (`Format_error "Chunk header too large"));
 	  );
-	  IFDEF Testing THEN self # case "accept_body_chunked/restart" ELSE () END;
+	  #ifdef Testing
+             self # case "accept_body_chunked/restart";
+          #endif
 	  `Restart_with(self # accept_body_chunked req_h resp 0)
 
   method private accept_body_chunked_contents req_h resp pos remaining_length () : cont =
     (* Read the chunk body at [pos], at most [remaining_length] bytes *)
-    IFDEF Testing THEN self # case "accept_body_chunked_contents" ELSE () END;
+    #ifdef Testing
+      self # case "accept_body_chunked_contents";
+    #endif
     let l = Netbuffer.length recv_buf in
     let s = Netbuffer.unsafe_buffer recv_buf in
     if remaining_length > 0 then (
-      IFDEF Testing THEN self # case "accept_body_chunked_contents/data" ELSE () END;
+      #ifdef Testing
+         self # case "accept_body_chunked_contents/data";
+      #endif
       (* There are still data to read *)
       let have_length = l - pos in
       let take_length = min have_length remaining_length in
@@ -1213,43 +1437,63 @@ object(self)
 	`Continue
 	  (self # accept_body_chunked_contents req_h resp (pos+take_length) 0)
       else (
-	IFDEF Testing THEN self # case "accept_body_chunked_contents/exceeded" ELSE () END;
+	#ifdef Testing
+          self # case "accept_body_chunked_contents/exceeded";
+        #endif
 	(* We hit the buffer boundary. Delete the buffer *)
 	if recv_eof then (
-	  IFDEF Testing THEN self # case "accept_body_chunked_contents/eof" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_contents/eof";
+          #endif
 	  raise(Bad_request `Unexpected_eof);
 	);
 	Netbuffer.clear recv_buf;
-	IFDEF Testing THEN self # case "accept_body_chunked_contents/restart" ELSE () END;
+	#ifdef Testing
+           self # case "accept_body_chunked_contents/restart";
+        #endif
 	`Restart_with 
 	  (self # accept_body_chunked_contents req_h resp 0 rem_length')
       )
     ) else (
-      IFDEF Testing THEN self # case "accept_body_chunked_contents/end" ELSE () END;
+      #ifdef Testing
+        self # case "accept_body_chunked_contents/end";
+      #endif
       (* End of chunk reached. There must a (single) line end at the end of the chunk *)
       if (l > pos && s.[pos] = '\010') then (
-	IFDEF Testing THEN self # case "accept_body_chunked_contents/lf" ELSE () END;
+	#ifdef Testing
+          self # case "accept_body_chunked_contents/lf";
+        #endif
 	`Continue(self # accept_body_chunked req_h resp (pos+1))
       )
       else
 	if (l > pos+1 && s.[pos] = '\013' && s.[pos+1] = '\010') then (
-	  IFDEF Testing THEN self # case "accept_body_chunked_contents/crlf" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_contents/crlf";
+          #endif
 	  `Continue(self # accept_body_chunked req_h resp (pos+2))
 	)
 	else
 	  if l > pos+1 then (
-	    IFDEF Testing THEN self # case "accept_body_chunked_contents/no_eol" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_chunked_contents/no_eol";
+            #endif
 	    raise (Bad_request (`Format_error "Chunk not followed by line terminator"))
 	  )
 	  else (
-	    IFDEF Testing THEN self # case "accept_body_chunked_contents/e_exceeded" ELSE () END;
+	    #ifdef Testing
+               self # case "accept_body_chunked_contents/e_exceeded";
+            #endif
 	    (* We hit the buffer boundary *)
 	    if recv_eof then (
-	      IFDEF Testing THEN self # case "accept_body_chunked_contents/e_eof" ELSE () END;
+	      #ifdef Testing
+                self # case "accept_body_chunked_contents/e_eof";
+              #endif
 	      raise(Bad_request `Unexpected_eof);
 	    );
 	    Netbuffer.delete recv_buf 0 pos;
-	    IFDEF Testing THEN self # case "accept_body_chunked_contents/e_restart" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_chunked_contents/e_restart";
+            #endif
 
 	    `Restart_with (self # accept_body_chunked_contents req_h resp 0 0)
 	  )
@@ -1257,14 +1501,18 @@ object(self)
     
   method private accept_body_chunked_end req_h resp pos () : cont =
     (* Read the trailer *)
-    IFDEF Testing THEN self # case "accept_body_chunked_end" ELSE () END;
+    #ifdef Testing
+      self # case "accept_body_chunked_end";
+    #endif
     let l = Netbuffer.length recv_buf in
     let s = Netbuffer.unsafe_buffer recv_buf in
     let config_max_trailer_length = max 2 (config # config_max_trailer_length) in
     try
       (* Check if there is a trailer *)
       if l > pos && s.[pos] = '\010' then (
-	IFDEF Testing THEN self # case "accept_body_chunked_end/lf" ELSE () END;
+	#ifdef Testing
+          self # case "accept_body_chunked_end/lf";
+        #endif
 	self # push_recv (`Req_end, 0);
 	pipeline_len <- pipeline_len + 1;
 	if resp#state = `Inhibited then
@@ -1273,7 +1521,9 @@ object(self)
       )
       else 
 	if l > pos+1 && s.[pos] = '\013' && s.[pos+1] = '\010' then (
-	  IFDEF Testing THEN self # case "accept_body_chunked_end/crlf" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_end/crlf";
+          #endif
 	  self # push_recv (`Req_end, 0);
 	  pipeline_len <- pipeline_len + 1;
 	  if resp#state = `Inhibited then
@@ -1281,23 +1531,31 @@ object(self)
 	  `Continue(self # accept_header (pos+2))
 	)
 	else (
-	  IFDEF Testing THEN self # case "accept_body_chunked_end/trailer" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_end/trailer";
+          #endif
 	  (* Assume there is a trailer. *)
 	  let trailer_end = http_find_double_line_start s pos (l-pos) in
               (* or Buf_exceeded *)
-	  IFDEF Testing THEN self # case "accept_body_chunked_end/tr_found" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_end/tr_found";
+          #endif
 	  (* Now we are sure there is a trailer! *)
 	  if trailer_end - pos > config_max_trailer_length then (
-	    IFDEF Testing THEN self # case "accept_body_chunked_end/tr_long" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_chunked_end/tr_long";
+            #endif
 	    raise(Bad_request (`Format_error "Trailer too long"));
 	  );
 	  let ch = new Netchannels.input_string 
 		     ~pos:pos ~len:(trailer_end - pos) s in
 	  let str = new Netstream.input_stream ch in
 	  let req_tr =
-	    try Netmime.read_mime_header str 
+	    try Netmime_channels.read_mime_header str 
 	    with Failure _ -> 
-	      IFDEF Testing THEN self # case "accept_body_chunked_end/bad_tr" ELSE () END;
+	      #ifdef Testing
+                self # case "accept_body_chunked_end/bad_tr";
+              #endif
 	      raise(Bad_request `Bad_trailer) in
 	  self # push_recv (`Req_trailer req_tr, trailer_end-pos);
 	  self # push_recv (`Req_end, 0);
@@ -1309,17 +1567,25 @@ object(self)
 
     with
 	Buffer_exceeded ->
-	  IFDEF Testing THEN self # case "accept_body_chunked_end/exceeded" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_end/exceeded";
+          #endif
 	  if recv_eof then (
-	    IFDEF Testing THEN self # case "accept_body_chunked_end/eof" ELSE () END;
+	    #ifdef Testing
+              self # case "accept_body_chunked_end/eof";
+            #endif
 	    raise(Bad_request `Unexpected_eof);
 	  );
 	  if l-pos > config_max_trailer_length then (
-	    IFDEF Testing THEN self # case "accept_body_chunked_end/tr_long" ELSE () END;
+	    #ifdef Testing 
+              self # case "accept_body_chunked_end/tr_long";
+            #endif
 	    raise(Bad_request (`Format_error "Trailer too long"));
 	  );
 	  Netbuffer.delete recv_buf 0 pos;
-	  IFDEF Testing THEN self # case "accept_body_chunked_end/restart" ELSE () END;
+	  #ifdef Testing
+            self # case "accept_body_chunked_end/restart";
+          #endif
 	  `Restart_with (self # accept_body_chunked_end req_h resp 0)
 
   (* ---- Process responses ---- *)
@@ -1339,7 +1605,12 @@ object(self)
 	  | `Resp_wire_data (s,pos,len) ->
 	      (* Try to write: *)
 	      dlogr (fun () -> sprintf "FD %Ld: send" fdi);
-	      let n = Unix.send fd s pos len [] in  (* or Unix.error *)
+	      let n =
+                match tls with
+                  | None ->
+                      Unix.send fd s pos len []  (* or Unix.error *)
+                  | Some t ->
+                      Netsys_tls.send t s pos len in
 	      dlogr (fun () -> sprintf "FD %Ld: sent %d bytes" fdi n);
 	      (* Successful. Advance by [n] *)
 	      resp # advance n;
@@ -1350,8 +1621,11 @@ object(self)
 	      pipeline_len <- pipeline_len - 1;
 	      resp # set_state `Processed;
 	      (* Check if we have to close the connection: *)
-	      if resp # close_connection then
-		self # shutdown_sending();
+	      if resp # close_connection then (
+                match tls with
+                  | None -> self # shutdown_sending()
+                  | Some _ -> tls_shutdown <- true
+              );
 	      (* Continue with the next response, if any, and if possible: *)
 	      let next_resp = Queue.take resp_queue in
 	      next_resp # set_state `Active;   (* ... unless dropped *)
@@ -1365,7 +1639,8 @@ object(self)
       with
 	| Send_queue_empty ->
 	    ()    (* nothing to do *)
-	| Unix.Unix_error((Unix.EAGAIN | Unix.EWOULDBLOCK),_,_) ->
+	| Unix.Unix_error((Unix.EAGAIN | Unix.EWOULDBLOCK),_,_)
+        | Netsys_types.EAGAIN_WR ->
 	    ()    (* socket not ready *)
 	| Unix.Unix_error(Unix.EINTR,_,_) ->
 	    ()    (* Signal happened, try later again *)
@@ -1377,6 +1652,12 @@ object(self)
 	    resp # set_state `Error;
 	    ignore(Queue.take resp_queue);
 	    self # abort (`Unix_error e)
+        | Netsys_types.EAGAIN_RD ->
+            assert false   (* not possible here *)
+        | Netsys_types.TLS_error code as e ->
+	    dlogr (fun () -> sprintf "FD %Ld: rev TLS_ERROR %s" fdi
+                                     (Netexn.to_string e));
+            self # abort (`TLS_error(code,tls_message code))
 
   method private drop_remaining_responses() =
     (* Set the state to [`Dropped] for all responses in the [resp_queue]: *)
@@ -1428,7 +1709,8 @@ object(self)
     dlogr (fun () -> sprintf "FD %Ld: shutdown" fdi);
     self # stop_input_acceptor();
     self # drop_remaining_responses();
-    self # shutdown_sending()
+    tls_shutdown <- true;
+    if tls = None then self # shutdown_sending()
 
   method private shutdown_sending() =
     dlogr (fun () -> sprintf "FD %Ld: shutdown_sending" fdi);
@@ -1484,6 +1766,7 @@ object(self)
 	     sprintf "FD %Ld: abort %s" fdi (string_of_fatal_error err));
     need_linger <- false;
     self # shutdown();
+    tls_shutdown_done <- true; (* don't do the TLS shutdown protocol *)
     let err' =
       if err=`Broken_pipe && config#config_suppress_broken_pipe then
 	`Broken_pipe_ignore
@@ -1499,17 +1782,49 @@ object(self)
 
 
   method do_input =
-    not recv_eof && 
-    (pipeline_len <= config#config_limit_pipeline_length) &&
-    (recv_queue_byte_size <= config#config_limit_pipeline_size)
+    match override_dir with
+      | None ->
+          not recv_fd_eof &&    (* CHECK: can we get further alerts after TLS END ? *)
+            (pipeline_len <= config#config_limit_pipeline_length) &&
+              (recv_queue_byte_size <= config#config_limit_pipeline_size)
+      | Some `R ->
+          true
+      | Some `W ->
+          false
 
   method do_output =
-    not (Queue.is_empty resp_queue) &&
-    not ((Queue.peek resp_queue) # send_queue_empty)
+    match override_dir with
+      | None ->
+          self # resp_queue_filled
+      | Some `R ->
+          false
+      | Some `W ->
+          true
+
+  method resp_queue_filled =
+    ( not (Queue.is_empty resp_queue) &&
+        not ((Queue.peek resp_queue) # send_queue_empty)
+    ) ||
+    ( tls_shutdown && not tls_shutdown_done )
+
 
   method need_linger = 
-    need_linger
+    (* TLS: never need lingering close, as TLS alerts indicate the end of the
+       data stream
+     *)
+    tls = None && need_linger
 
+
+  method tls_session_props =
+    if not tls_handshake && tls_session_props = None then (
+      match tls with
+        | Some t ->
+             tls_session_props <- 
+               Some (Nettls_support.get_tls_session_props t)
+        | None ->
+             ()
+    );
+    tls_session_props
 end
 
 
@@ -1563,6 +1878,7 @@ let default_http_protocol_config =
       method config_limit_pipeline_size = 65536
       method config_announce_server = `Ocamlnet
       method config_suppress_broken_pipe = false
+      method config_tls = None
     end
   )
 
@@ -1579,6 +1895,7 @@ class modify_http_protocol_config
         ?config_limit_pipeline_size
         ?config_announce_server
         ?config_suppress_broken_pipe 
+        ?config_tls
         (config : http_protocol_config) : http_protocol_config =
   let config_max_reqline_length = 
     override config#config_max_reqline_length config_max_reqline_length in
@@ -1594,6 +1911,8 @@ class modify_http_protocol_config
     override config#config_announce_server config_announce_server in
   let config_suppress_broken_pipe =
     override config#config_suppress_broken_pipe config_suppress_broken_pipe in
+  let config_tls =
+    override config#config_tls config_tls in
 object
   method config_max_reqline_length = config_max_reqline_length
   method config_max_header_length = config_max_header_length
@@ -1602,4 +1921,5 @@ object
   method config_limit_pipeline_size = config_limit_pipeline_size
   method config_announce_server = config_announce_server
   method config_suppress_broken_pipe = config_suppress_broken_pipe
+  method config_tls = config_tls
 end
