@@ -1,4 +1,4 @@
-(* $Id: nethttpd_kernel.mli,v 1.2 2005/07/25 22:36:41 stolpmann Exp $
+(* $Id: nethttpd_kernel.mli 2195 2015-01-01 12:23:39Z gerd $
  *
  *)
 
@@ -18,7 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with WDialog; if not, write to the Free Software
+ * along with Nethttpd; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *)
 
@@ -38,15 +38,19 @@ open Nethttpd_types
 
 type fatal_error =
     [ `Broken_pipe
+    | `Broken_pipe_ignore
     | `Message_too_long
     | `Timeout
     | `Unix_error of Unix.error
+    | `TLS_error of string * string
     | `Server_error
     ]
     (** These are the serious protocol violations after that the daemon stops
       * any further processing.
       *
       * Note that [`Timeout] refers to a timeout in the middle of a request.
+      * [`Broken_pipe_ignore] is the "harmless" version of [`Broken_pipe]
+      * (see [config_suppress_broken_pipe]).
       *
       * Long messages are fatal because it is suspected that they are denial
       * of service attacks. The kernel generates [`Message_too_long] only for
@@ -55,6 +59,9 @@ type fatal_error =
       * Fatal server errors can happen when exceptions are not properly handled.
       * As last resort the HTTP daemon closes the connection without notifying
       * the client.
+      *
+      * [`TLS_error] means any error on the TLS level. The two strings identify
+      * the problem as for {!Netsys_tls.Error}.
      *)
 
 val string_of_fatal_error : fatal_error -> string
@@ -147,6 +154,12 @@ type front_token =
 
 exception Send_queue_empty
 
+
+type announcement =
+    [`Ignore | `Ocamlnet | `Ocamlnet_and of string | `As of string ]
+  (** See config *)
+
+
 (** Encapsultation of the HTTP response for a single request *)
 class type http_response  =
 object
@@ -238,6 +251,9 @@ object
       * fail, it is called in situations that make it hard to recover from errors.
      *)
 
+  method body_size : int64
+    (** Accumulated size of the response body *)
+
   (** These methods must only be called by the HTTP protocol processor: *)
 
   method set_state : resp_state -> unit
@@ -251,17 +267,21 @@ object
       * front token is modified such that the position is advanced by
       * [n], and the length is reduced by [n].
      *)
+
 end
 
-class http_response_impl : ?close:bool -> ?suppress_body:bool -> protocol -> http_response
+class http_response_impl : ?close:bool -> ?suppress_body:bool -> int64 -> protocol -> announcement -> http_response
   (** Exported for debugging and testing only *)
 
-val send_static_response : http_response -> 
-                           http_status -> http_header option -> string -> unit
+val send_static_response : 
+      http_response -> 
+      http_status -> http_header option -> string -> unit
   (** Sends the string argument as response body, together with the given status and
     * the header (optional). Response header fields are set as follows:
     * - The [Content-Length] is set to the length of the string.
     * - The [Content-Type] is set to "text/html" unless given by the header.
+    * If the header object is passed in, these modifications are done 
+    * directly in this object as side effect.
    *)
 
 val send_file_response : http_response -> 
@@ -277,6 +297,9 @@ val send_file_response : http_response ->
     * Note that [Content-Range] is not set automatically, even if the file is only
     * partially transferred.
     *
+    * If the header object is passed in, these modifications are done 
+    * directly in this object as side effect.
+    *
     * The function does not send the file immediately, but rather sets the [http_response]
     * object up that the next chunk of the file is added when the send queue becomes
     * empty. This file will be closed when the transfer is done.
@@ -301,7 +324,7 @@ type req_token =
       *   the corresponding [http_response] object is returned which must
       *   be used to transmit the response.
       * - [`Req_expect_100_continue] is generated when the client expects that the
-      *   server sends a "100 Contnue" response (or a final status code) now.
+      *   server sends a "100 Continue" response (or a final status code) now.
       *   One should add [`Resp_info_line resp_100_continue] to the send queue
       *   if the header is acceptable, or otherwise generate an error response. In any
       *   case, the rest of the request must be read until [`Req_end].
@@ -325,6 +348,9 @@ type req_token =
       * in time.
      *)
 
+val string_of_req_token : req_token -> string
+  (** For debugging *)
+
 exception Recv_queue_empty
 
 (** Configuration values for the HTTP kernel *)
@@ -358,6 +384,60 @@ object
       * receiver completely (not recommended).
      *)
 
+  method config_announce_server : announcement
+    (** Whether to set the [Server] header:
+     * - [`Ignore]: The kernel does not touch the [Server] header.
+     * - [`Ocamlnet]: Announce this web server as "Ocamlnet/<version>"
+     * - [`Ocamlnet_and s]: Announce this web server as [s] and append
+     *   the Ocamlnet string.
+     * - [`As s]: Announce this web server as [s]
+     *)
+
+  method config_suppress_broken_pipe : bool
+    (** Whether to suppress [`Broken_pipe] errors. Instead 
+      * [`Broken_pipe_ignore] is reported.
+     *)
+
+  method config_tls : Netsys_crypto_types.tls_config option
+    (** If set, the server enables TLS with the arugment config *)
+
+end
+
+
+val default_http_protocol_config : http_protocol_config 
+  (** Default config:
+      - [config_max_reqline_length = 32768]
+      - [config_max_header_length = 65536]
+      - [config_max_trailer_length = 32768]
+      - [config_limit_pipeline_length = 5]
+      - [config_limit_pipeline_size = 65536]
+      - [config_announce_server = `Ocamlnet]
+      - [config_suppress_broken_pipe = false]
+      - [config_tls = None]
+   *)
+
+class modify_http_protocol_config : 
+        ?config_max_reqline_length:int ->
+        ?config_max_header_length:int ->
+        ?config_max_trailer_length:int ->
+        ?config_limit_pipeline_length:int ->
+        ?config_limit_pipeline_size:int ->
+        ?config_announce_server:announcement ->
+        ?config_suppress_broken_pipe:bool ->
+        ?config_tls:Netsys_crypto_types.tls_config option ->
+        http_protocol_config -> http_protocol_config 
+  (** Modifies the passed config object as specified by the optional
+      arguments
+   *)
+
+(** Allows it to set hooks for {!Nethttpd_kernel.http_protocol} *)
+class type http_protocol_hooks =
+object
+  method tls_set_cache : store:(string -> string -> unit) ->
+                         remove:(string -> unit) ->
+                         retrieve:(string -> string) ->
+                            unit
+    (** Sets the functions for accessing the session cache *)
 end
 
 
@@ -439,6 +519,9 @@ object
     * 400 error response.
    *)
 
+  method hooks : http_protocol_hooks
+    (** Set hooks *)
+
   method cycle : ?block:float -> unit -> unit
     (** Looks at the file descriptor. If there is data to read from the descriptor,
       * and there is free space in the input buffer, additional data is read into
@@ -501,6 +584,8 @@ object
 
   method shutdown : unit -> unit
     (** Shuts the socket down. Note: the descriptor is not closed.
+
+        TLS: You need to further [cycle] until XXX.
      *)
 
   method timeout : unit -> unit
@@ -524,12 +609,22 @@ object
   method do_output : bool
     (** Returns [true] iff the protocol engine has data to output to the socket *)
 
+  method resp_queue_filled : bool
+    (** Whether there is data to send in the internal output queue. If
+        TLS is enabled, this is not always the same as [do_output].
+     *)
+
   method need_linger : bool
     (** Returns [true] when a lingering close operation is needed to reliably shut
       * down the socket. In many cases, this expensive operation is not necessary.
       * See the class [lingering_close] below.
      *)
 
+  method tls_session_props : Nettls_support.tls_session_props option
+    (** If TLS is enabled, this returns the session properties. These
+        are first available after the TLS handshake.
+     *)
+   
   method config : http_protocol_config
     (** Just returns the configuration *)
 
@@ -541,8 +636,10 @@ object
 end
 
 
-(** Closes a file descriptor using the "lingering close" algorithm. *)
-class lingering_close : Unix.file_descr ->
+(** Closes a file descriptor using the "lingering close" algorithm.
+    The optional [preclose] function is called just before [Unix.close].
+ *)
+class lingering_close : ?preclose:(unit->unit) -> Unix.file_descr ->
 object
   (** Closes a file descriptor using the "lingering close" algorithm
     * 
@@ -562,3 +659,12 @@ object
   method fd : Unix.file_descr
     (** The file descriptor *)
 end
+
+
+(** {1 Debugging} *)
+
+module Debug : sig
+  val enable : bool ref
+    (** Enables {!Netlog}-style debugging of this module
+     *)
+end
