@@ -1,4 +1,4 @@
-(* $Id: nethttpd_plex.ml 2195 2015-01-01 12:23:39Z gerd $ *)
+(* $Id: nethttpd_plex.ml 2245 2015-06-21 14:12:41Z gerd $ *)
 
 open Nethttpd_services
 open Netplex_types
@@ -49,8 +49,9 @@ class nethttpd_processor ?(hooks = new Netplex_kit.empty_processor_hooks())
 object(self)
   inherit Netplex_kit.processor_hooks_delegation hooks
 
-  method post_add_hook _ ctrl =
-    ctrl # add_plugin Netplex_sharedvar.plugin
+  method post_add_hook sockserv ctrl =
+    ctrl # add_plugin Netplex_sharedvar.plugin;
+    hooks # post_add_hook sockserv ctrl
 
   method process ~when_done (container : Netplex_types.container) fd proto =
     let config = mk_config container in
