@@ -1,4 +1,4 @@
-(* $Id: nethttpd_services.ml 1642 2011-07-20 20:46:25Z gerd $
+(* $Id: nethttpd_services.ml 2195 2015-01-01 12:23:39Z gerd $
  *
  *)
 
@@ -1147,7 +1147,8 @@ object(self)
       if req_path <> "" && req_path.[ String.length req_path - 1 ] <> '/' then (
 	let h = new Netmime.basic_mime_header
 	  [ "Location", 
-	    sprintf "http://%s%s%s/"
+	    sprintf "%s://%s%s%s/"
+              (if env#cgi_https then "https" else "http")
 	      env#cgi_server_name
 	      ( match env#cgi_server_port with 
 		  | Some p -> ":" ^ string_of_int p
