@@ -1,4 +1,4 @@
-(* $Id: netmcore.mli 1713 2012-02-16 21:32:41Z gerd $ *)
+(* $Id: netmcore.mli 1850 2013-06-02 23:10:26Z gerd $ *)
 
 (** Multi-processing for compute jobs
 
@@ -8,6 +8,10 @@
     Instead, independent processes are created, and the communication
     between processes is made as cheap and unproblematic as possible.
 
+    Please read {!Netmcore_basics} for an introduction into using
+    this module. There is also {!Netmcore_tut} with more advanced
+    techniques.
+
     Before using this library, it is required to call {!Netmcore.startup}.
     This turns the current process into the master process. The master
     process has only a supervising function, and is responsible for
@@ -27,6 +31,28 @@
     no such problem in the worker processes. In this library,
     however, nothing is done to ease the multi-threaded life, so you
     should carefully check the compatibility first.
+
+    {!Netmcore} controls the lifetime of certain system resources in
+    addition to worker processes:
+    - Temporary files
+    - Shared memory objects
+    - Named semaphores
+
+    These objects have kernel persistence, and continue to exist when
+    the program ends. Because of this, there are two mechanisms to
+    control the lifetime, and to delete these objects finally: First,
+    if all using processes terminate normally, an object is deleted.
+    Second, in order to also catch abnormal terminations, these objects
+    are also managed by storing their names into an external file
+    "netplex.pmanage" (in the socket directory). When the program is
+    started the next time with the same socket directory, the objects
+    from the previous run are automatically deleted. The second mechanism
+    can be disabled by setting the
+    - [disable_pmanage] parameter for the {!Netmcore.startup} call, or by
+      setting the
+    - [unlink] parameter in the same call. This second option only disables
+      the deletion, but the management file is updated.
+
  *)
 
 
@@ -49,7 +75,8 @@ type process_id =
 
 
 type compute_resource_type =
-    [ `File | `Posix_shm | `Posix_shm_preallocated 
+    [ `File | `Posix_shm | `Posix_shm_sc | `Posix_shm_preallocated 
+    | `Posix_shm_preallocated_sc
     | `Posix_sem | `Fork_point | `Join_point
     ]
 
@@ -61,18 +88,25 @@ type inherit_request =
 type compute_resource_repr =
     [ `File of string
     | `Posix_shm of string
+    | `Posix_shm_sc of string * Netsys_sem.prefix
     | `Posix_shm_preallocated of string * Netsys_mem.memory
+    | `Posix_shm_preallocated_sc of string * Netsys_mem.memory * 
+                                       Netsys_sem.container
     | `Posix_sem of string
     | `Fork_point of (inherit_request * Netplex_encap.encap -> process_id)
     | `Join_point of (process_id -> Netplex_encap.encap option)
     ]
   (** Centrally managed resources include:
-      - [`File name]: Files [name] (absolute name)
+      - [`File name]: (Temporary) file [name] (absolute name)
       - [`Posix_shm name]: Shared memory objects with [name]
+      - [`Posix_shm_sc(name,p)]: Shared memory objects with [name], and
+        attached container for semaphores with prefix [p]
       - [`Posix_shm_preallocated(name,m)]: Shared memory objects already
         allocated by the master process. These objects are passed over
         to the worker processes by inheritance, and are always mapped at
         the same address. [m] is the bigarray mapping the object.
+      - [`Posix_shm_preallocated_sc(name,m.p)]: Same, plus attached
+        container for semaphores with prefix [p]
       - [`Posix_sem name]: Semaphores with [name]
       - [`Fork_point(inh,f)]: Fork points where [let pid=f arg] fork a new process
         with argument [arg]. [pid] is the process identifier. The list [inh]
@@ -81,6 +115,10 @@ type compute_resource_repr =
         the process [pid] terminates. If [res] is non-[None] it is the
         result value. If it is [None], no result was passed back
         (including all pathological cases like crashes)
+
+      Except fork and join points, these resources are also added to
+      the pmanage object of Netplex (see {!Netplex_cenv.pmanage} and
+      {!Netsys_pmanage}).
    *)
 
 exception No_resource of res_id
@@ -196,9 +234,20 @@ val start : ?inherit_resources:inherit_request ->
 
 val join : res_id -> process_id -> Netplex_encap.encap option
   (** [let res_opt = join join_point pid]: Waits until the process [pid]
-      is done, and returns the result value if any.
+      is done, and returns the result value if any. You can only join
+      (or [join_nowait]) once per process.
 
       Raises [No_resource] if there is no such resource.
+
+      It is not possible to call [join] from the master process (as this
+      process must not block). You can, however, get the result of an
+      already terminated process with [join_nowait], e.g. after
+      {!Netmcore.run} returned.
+   *)
+
+val join_nowait : res_id -> process_id -> Netplex_encap.encap option
+  (** [let res_opt = join_nowait join_point pid]: Checks if the process [pid]
+      is done, and returns the result value if so. Otherwise return [None].
    *)
 
 (** {2 Managing resources from worker processes} *)
@@ -231,11 +280,18 @@ val get_file : res_id -> string
 val manage_shm : string -> compute_resource
   (** hands over a named shm object to the manager *)
 
+val manage_shm_sc : string -> Netsys_sem.container -> compute_resource
+  (** hands over a named shm object plus semaphore container to the manager *)
+
 val get_shm : res_id -> string
   (** Gets the shm object with this ID (or raises [No_resource]). As
       for [get_resource] the shm object is marked as being used by the process.
    *)
 
+val get_sem_container : res_id -> Netsys_sem.container
+  (** Get the semaphore container *)
+
+
 (** Shared memory objects can be created with {!Netsys_posix.shm_create},
     and opened with {!Netsys_posix.shm_open}. 
  *)
@@ -263,6 +319,13 @@ val create_preallocated_shm :
       memory area.
    *)
 
+val create_preallocated_shm_sc : 
+     ?value_area:bool -> string -> int -> 
+         (res_id * string * Netsys_sem.container)
+  (** Same as [create_preallocated_shm] with the extension that 
+      a semaphore container is also allocated and returned
+   *)
+
 val manage_sem : string -> compute_resource
   (** hands over a named semaphore to the manager *)
 
@@ -293,8 +356,10 @@ val add_plugins : Netplex_types.controller -> unit
 val startup : socket_directory:string ->
               ?pidfile:string ->
               ?init_ctrl:(Netplex_types.controller -> unit) ->
-              first_process:(unit -> process_id) ->
-              unit ->
+              ?disable_pmanage:bool ->
+              ?no_unlink:bool ->
+              first_process:('a -> process_id) ->
+              'a ->
                 unit
   (** This function makes the current process the master process.
       It starts immediately a new worker process, called the 
@@ -303,7 +368,7 @@ val startup : socket_directory:string ->
       shut down (which may lead to killing the remaining processes,
       following the usual shutdown procedure).
 
-      The first process is created by calling [first_process()] at the
+      The first process is created by calling [first_process] at the
       right moment. This function normally just invokes [start].
 
       Passing a [socket_directory] is mandatory. This directory will
@@ -312,6 +377,33 @@ val startup : socket_directory:string ->
 
       [pidfile]: If passed, the Unix PID of the master process is written
       to this file.
+
+      [disable_pmanage]: Disable that persistent kernel objects are
+      managed via the "netplex.pmanage" file in the socket directory.
+
+      [no_unlink]: Disable that old persistent kernel objects are deleted at
+      startup. This may be useful when [startup] is called a second time
+      from the same program.
+   *)
+
+val run : socket_directory:string ->
+          ?pidfile:string ->
+          ?init_ctrl:(Netplex_types.controller -> unit) ->
+          ?disable_pmanage:bool ->
+          ?no_unlink:bool ->
+          first_process:('a -> process_id) ->
+          extract_result:(Netplex_types.controller -> process_id -> 'b) ->
+          'a ->
+            'b
+  (** This function fixes a design error in [startup], namely that it was
+      difficult to get the result of the process(es) back. Here, the
+      additional callback [extract_result] can be used to retrieve the
+      results from processes, especially from the first process.
+
+      You need to call {!Netmcore.join_nowait} to get the results from
+      processes.
+
+      The result of [extract_result] is the result of [run].
    *)
 
 val destroy_resources : unit -> unit
