@@ -1,4 +1,4 @@
-(* $Id: netmcore_camlbox.ml 1756 2012-03-24 20:52:25Z gerd $ *)
+(* $Id: netmcore_camlbox.ml 1803 2012-10-18 23:44:51Z gerd $ *)
 
 open Netcamlbox
 open Netmcore
@@ -19,10 +19,18 @@ let create_camlbox prefix n size =
     (box, res#id)
   with
     | error ->
-	if !fd_open then Unix.close fd;
+	if !fd_open then ( 
+          (* Apparently, "Unix.close fd" is rejected on OS X when the shm
+             has not been ftruncated. So just try that.
+          *)
+          (try Unix.ftruncate fd 0 with _ -> ());
+	  Unix.close fd;
+	);
 	raise error
 
 
+
+
 let lookup_camlbox_address res_id =
   let name = get_shm res_id in
   assert(name.[0] = '/');
