@@ -1,4 +1,4 @@
-(* $Id: netchannels.ml 1717 2012-02-20 17:31:58Z gerd $
+(* $Id: netchannels.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -346,9 +346,10 @@ object
 end
 
 
-class input_netbuffer b : nb_in_obj_channel =
+class input_netbuffer ?(keep_data=false) b : nb_in_obj_channel =
 object (self)
   val mutable b = b
+  val mutable offset = 0
   val mutable eof = false
   val mutable closed = false
   val mutable ch_pos = 0
@@ -358,16 +359,19 @@ object (self)
 
   method input buf pos len =
     if closed then self # complain_closed();
-    if pos < 0 || len < 0 || pos+len > String.length buf then
+    if pos < 0 || len < 0 || pos > String.length buf - len then
       invalid_arg "input";
 
-    let n = min len (Netbuffer.length b) in
+    let n = min len (Netbuffer.length b - offset) in
     if n = 0 && len>0 then begin
       if eof then raise End_of_file else raise Buffer_underrun
     end
     else begin
-      Netbuffer.blit b 0 buf pos n;
-      Netbuffer.delete b 0 n;
+      Netbuffer.blit b offset buf pos n;
+      if keep_data then
+        offset <- offset + n
+      else
+        Netbuffer.delete b 0 n;
       ch_pos <- ch_pos + n;
       n
     end
@@ -394,20 +398,27 @@ object (self)
   method input_line() =
     if closed then self # complain_closed();
     try
-      let k = Netbuffer.index_from b 0 '\n' in
+      let k = Netbuffer.index_from b offset '\n' in
       (* CHECK: Are the different end of line conventions important here? *)
-      let line = Netbuffer.sub b 0 k in
-      Netbuffer.delete b 0 (k+1);
+      let line = Netbuffer.sub b offset (k - offset) in
+      if keep_data then
+        offset <- offset + k + 1
+      else
+        Netbuffer.delete b 0 (k+1);
       ch_pos <- ch_pos + k + 1;
       line
     with
 	Not_found ->
 	  if eof then begin
-	    if Netbuffer.length b = 0 then raise End_of_file;
+            let n = Netbuffer.length b - offset in
+	    if n=0 then raise End_of_file;
 	    (* Implicitly add linefeed at the end of the file: *)
-	    let line = Netbuffer.contents b in
-	    Netbuffer.clear b;
-	    ch_pos <- ch_pos + (Netbuffer.length b);
+	    let line = Netbuffer.sub b offset n in
+            if keep_data then
+              offset <- offset + n
+            else
+	      Netbuffer.clear b;
+	    ch_pos <- ch_pos + n;
 	    line
 	  end
 	  else raise Buffer_underrun
@@ -431,8 +442,8 @@ end
 ;;
 
 
-let create_input_netbuffer b =
-  let ch = new input_netbuffer b in
+let create_input_netbuffer ?keep_data b =
+  let ch = new input_netbuffer ?keep_data b in
   (ch :> in_obj_channel), (ch # shutdown)
 ;;
 
@@ -1340,8 +1351,39 @@ let lift_out ?(buffered=true) ?buffer_size ?pass_through (x : lift_out_arg) =
 
 (************************* raw channels *******************************)
 
-class input_descr_prelim ?(blocking=true) ?(start_pos_in = 0) fd =
-  let fd_style = Netsys.get_fd_style fd in
+
+let norestart _ _ _ f arg =
+  try
+    f arg
+  with
+    | Unix.Unix_error(Unix.EAGAIN,_,_)
+    | Unix.Unix_error(Unix.EWOULDBLOCK,_,_)
+    | Netsys_types.EAGAIN_RD
+    | Netsys_types.EAGAIN_WR ->
+         0
+
+
+let shutdown_fd mode fd_style fd =
+  try
+    ignore
+      (Netsys.restart_wait mode fd_style fd
+         (fun () ->
+            Netsys.gshutdown fd_style fd Unix.SHUTDOWN_ALL; 0
+         )
+         ()
+      )
+  with
+    | Netsys.Shutdown_not_supported -> ()
+    | Unix.Unix_error(Unix.EPERM, _, _) -> ()
+
+
+class input_descr_prelim ?(blocking=true) ?(start_pos_in = 0) ?fd_style fd =
+  let fd_style =
+    match fd_style with
+      | None -> Netsys.get_fd_style fd
+      | Some st -> st in
+  let wrapper =
+    if blocking then Netsys.restart_wait else norestart in
 object (self)
   val fd_in = fd
   val mutable pos_in = start_pos_in
@@ -1352,26 +1394,20 @@ object (self)
 
   method input buf pos len =
     if closed_in then self # complain_closed();
-    try
-      let n = Netsys.gread fd_style fd_in buf pos len in
-      pos_in <- pos_in + n;
-      if n=0 && len>0 then raise End_of_file;
-      n
-    with
-	Unix.Unix_error(Unix.EINTR,_,_) ->
-	  self # input buf pos len
-      | Unix.Unix_error(Unix.EAGAIN,_,_)
-      | Unix.Unix_error(Unix.EWOULDBLOCK,_,_) ->
-	  if blocking then (
-	    let _  = Netsys.restart 
-	      (Netsys.wait_until_readable fd_style fd) (-1.0) in
-	    self # input buf pos len
-	  )
-	  else 0
+    wrapper `R fd_style fd
+      (fun () ->
+         let n = Netsys.gread fd_style fd_in buf pos len in
+         pos_in <- pos_in + n;
+         if n=0 && len>0 then raise End_of_file;
+         n
+      )
+      ()
 	  
   
   method close_in () =
     if not closed_in then (
+      (* The gshutdown call only exists because of TLS: *)
+      shutdown_fd `R fd_style fd;
       Netsys.gclose fd_style fd_in; 
       closed_in <- true
     )
@@ -1383,13 +1419,18 @@ end
 ;;
 
 
-class input_descr ?blocking ?start_pos_in fd : raw_in_channel = 
-  input_descr_prelim ?blocking ?start_pos_in fd
+class input_descr ?blocking ?start_pos_in ?fd_style fd : raw_in_channel = 
+  input_descr_prelim ?blocking ?start_pos_in ?fd_style fd
 ;;
 
 
-class output_descr_prelim ?(blocking=true) ?(start_pos_out = 0) fd =
-  let fd_style = Netsys.get_fd_style fd in
+class output_descr_prelim ?(blocking=true) ?(start_pos_out = 0) ?fd_style fd =
+  let fd_style =
+    match fd_style with
+      | None -> Netsys.get_fd_style fd
+      | Some st -> st in
+  let wrapper =
+    if blocking then Netsys.restart_wait else norestart in
 object (self)
   val fd_out = fd
   val mutable pos_out = start_pos_out
@@ -1400,41 +1441,22 @@ object (self)
 
   method output buf pos len =
     if closed_out then self # complain_closed();
-    try
-      let n = Netsys.gwrite fd_style fd_out buf pos len in
-      pos_out <- pos_out + n;
-      n
-    with
-	Unix.Unix_error(Unix.EINTR,_,_) ->
-	  self # output buf pos len
-      | Unix.Unix_error(Unix.EAGAIN,_,_)
-      | Unix.Unix_error(Unix.EWOULDBLOCK,_,_) ->
-	  if blocking then (
-	    let _  = Netsys.restart 
-	      (Netsys.wait_until_writable fd_style fd) (-1.0) in
-	    self # output buf pos len
-	  )
-	  else
-	    0
+    wrapper `W fd_style fd
+      (fun () ->
+         let n = Netsys.gwrite fd_style fd_out buf pos len in
+         pos_out <- pos_out + n;
+         n
+      )
+      ()
   
   method close_out () =
     if not closed_out then (
-      ( try
-	  Netsys.gshutdown fd_style fd Unix.SHUTDOWN_SEND
-	with
-	  | Netsys.Shutdown_not_supported -> ()
-	  | Unix.Unix_error(Unix.EAGAIN, _, _) ->
-	      (* FIXME. We block here even when non-blocking semantics
-               is requested. We do this because most programmers would
-               be surprised to get EAGAIN when closing a channel.
-               Actually, this only affects Win32 output threads.
-	       *)
-	      let _  = Netsys.restart 
-		(Netsys.wait_until_writable fd_style fd) (-1.0) in
-	      Netsys.gshutdown fd_style fd Unix.SHUTDOWN_SEND
-	  | Unix.Unix_error(Unix.EPERM, _, _) ->
-	      ()
-      );
+      (* FIXME. We block here even when non-blocking semantics
+         is requested. We do this because most programmers would
+         be surprised to get EAGAIN when closing a channel.
+         Actually, this only affects Win32 output threads and TLS.
+       *)
+      shutdown_fd `W fd_style fd;
       Netsys.gclose fd_style fd_out; 
       closed_out <- true
     )
@@ -1449,35 +1471,34 @@ end
 ;;
 
 
-class output_descr ?blocking ?start_pos_out fd : raw_out_channel =
-  output_descr_prelim ?blocking ?start_pos_out fd 
+class output_descr ?blocking ?start_pos_out ?fd_style fd : raw_out_channel =
+  output_descr_prelim ?blocking ?start_pos_out ?fd_style fd 
 ;;
 
 
-class socket_descr ?blocking ?(start_pos_in = 0) ?(start_pos_out = 0) fd 
+class socket_descr ?blocking ?(start_pos_in = 0) ?(start_pos_out = 0) 
+                   ?fd_style fd 
       : raw_io_channel =
-  let fd_style = Netsys.get_fd_style fd in
+  let fd_style =
+    match fd_style with
+      | None -> Netsys.get_fd_style fd
+      | Some st -> st in
   let () =
     match fd_style with
       | `Recv_send _
       | `Recv_send_implied
-      | `W32_pipe -> ()
+      | `W32_pipe
+      | `TLS _ -> ()
       | _ ->
 	  failwith "Netchannels.socket_descr: This type of descriptor is \
                     unsupported"
   in
 object (self)
-  inherit input_descr_prelim ?blocking ~start_pos_in fd
-  inherit output_descr_prelim ?blocking ~start_pos_out fd 
+  inherit input_descr_prelim ?blocking ~start_pos_in ~fd_style fd
+  inherit output_descr_prelim ?blocking ~start_pos_out ~fd_style fd 
   
   method private gen_close cmd =
-    ( try
-	Netsys.gshutdown fd_style fd cmd
-      with
-	| Netsys.Shutdown_not_supported -> ()
-	| Unix.Unix_error(Unix.EAGAIN, _, _) -> assert false
-	| Unix.Unix_error(Unix.EPERM, _, _) -> ()
-    );
+    shutdown_fd `W fd_style fd;
     if cmd = Unix.SHUTDOWN_ALL then
       Netsys.gclose fd_style fd
 
