@@ -1,4 +1,4 @@
-(* $Id: netchannels.mli 1717 2012-02-20 17:31:58Z gerd $
+(* $Id: netchannels.mli 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -417,6 +417,7 @@ class input_string :
 
 
 val create_input_netbuffer :
+  ?keep_data:bool ->
   Netbuffer.t ->
     in_obj_channel   *   (* shutdown: *) (unit -> unit)
   (** Creates an input channel and a shutdown function for a netbuffer. 
@@ -432,6 +433,8 @@ val create_input_netbuffer :
    * If the netbuffer becomes empty, the input methods raise [Buffer_underrun]
    * when the EOF condition has not yet been set, and they raise
    * [End_of_file] when the EOF condition has been recorded.
+   *
+   * [keep_data]: do not delete read data from the buffer
    *)
 
 val lexbuf_of_in_obj_channel : in_obj_channel -> Lexing.lexbuf
@@ -802,6 +805,7 @@ class buffered_raw_out_channel :
 class input_descr :
   ?blocking:bool ->
   ?start_pos_in:int ->
+  ?fd_style:Netsys.fd_style ->
   Unix.file_descr ->
     raw_in_channel
   (** Creates a [raw_in_channel] for the passed file descriptor, which must
@@ -819,12 +823,15 @@ class input_descr :
    * possible to read from the (non-blocking) descriptor. Defaults to [true].
    * @param start_pos_in The position to which [pos_in] is initialized when
    * the channel is created, by default 0
+   * @param fd_style The descriptor style. If omitted, it is automatically
+   * determined if possible.
    *)
 
 
 class output_descr :
   ?blocking:bool ->
   ?start_pos_out:int ->
+  ?fd_style:Netsys.fd_style ->
   Unix.file_descr ->
     raw_out_channel
   (** Creates a [raw_out_channel] for the passed file descriptor, which must
@@ -842,12 +849,15 @@ class output_descr :
    * possible to write to the (non-blocking) descriptor. Defaults to [true].
    * @param start_pos_out The position to which [pos_out] is initialized when
    * the channel is created, by default 0
+   * @param fd_style The descriptor style. If omitted, it is automatically
+   * determined if possible.
    *)
 
 class socket_descr :
   ?blocking:bool ->
   ?start_pos_in:int ->
   ?start_pos_out:int ->
+  ?fd_style:Netsys.fd_style ->
   Unix.file_descr ->
     raw_io_channel
   (** Creates a [raw_io_channel] for the passed socket descriptor, which must
@@ -868,6 +878,8 @@ class socket_descr :
    * the channel is created, by default 0
    * @param start_pos_out The position to which [pos_out] is initialized when
    * the channel is created, by default 0
+   * @param fd_style The descriptor style. If omitted, it is automatically
+   * determined if possible.
    *)
 
 
