@@ -1,8 +1,29 @@
-(* $Id: netencoding.ml 1652 2011-08-03 21:50:30Z gerd $
+(* $Id: netencoding.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
 
+let hexdigit_uc =
+  [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7';
+     '8'; '9'; 'A'; 'B'; 'C'; 'D'; 'E'; 'F'; |]
+
+let hexdigit_lc =
+  [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7';
+     '8'; '9'; 'a'; 'b'; 'c'; 'd'; 'e'; 'f'; |]
+
+
+let to_hex ?(lc=false) s =
+  let hexdigit = if lc then hexdigit_lc else hexdigit_uc in
+  let l = String.length s in
+  let u = String.create (2*l) in
+  for k = 0 to l-1 do
+    let c = String.unsafe_get s k in
+    let j = k lsl 1 in
+    String.unsafe_set u j     hexdigit.(Char.code c lsr 4);
+    String.unsafe_set u (j+1) hexdigit.(Char.code c land 15);
+  done;
+  u
+
 
 module Base64 = struct
   let b64_pattern plus slash =
@@ -350,14 +371,14 @@ module Base64 = struct
       let c2 = next_char() in
       let c3 = next_char() in
 
-      if (c2 = '=' & c3 = '=') or (p_url & c2 = '.' & c3 = '.') then begin
+      if (c2 = '=' && c3 = '=') || (p_url && c2 = '.' && c3 = '.') then begin
 	let n0 = decode_char c0 in
 	let n1 = decode_char c1 in
 	let x0 = (n0 lsl 2) lor (n1 lsr 4) in
 	s.[ q ]   <- Char.chr x0;
       end
       else
-	if (c3 = '=') or (p_url & c3 = '.') then begin
+	if (c3 = '=') || (p_url && c3 = '.') then begin
 	  let n0 = decode_char c0 in
 	  let n1 = decode_char c1 in
 	  let n2 = decode_char c2 in
@@ -517,16 +538,12 @@ module QuotedPrintable = struct
     let t_len = count !line_length 0 0 in
     let t = String.create t_len in
     
-    let hexdigit =
-      [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7';
-	 '8'; '9'; 'A'; 'B'; 'C'; 'D'; 'E'; 'F'; |] in
-
     let k = ref 0 in
 
     let add_quoted c =
       t.[ !k ]   <- '=';
-      t.[ !k+1 ] <- hexdigit.( Char.code c lsr 4 );
-      t.[ !k+2 ] <- hexdigit.( Char.code c land 15 )
+      t.[ !k+1 ] <- hexdigit_uc.( Char.code c lsr 4 );
+      t.[ !k+2 ] <- hexdigit_uc.( Char.code c land 15 )
     in
 
     let add_soft_break() =
@@ -638,7 +655,7 @@ module QuotedPrintable = struct
 
   let decode_substring s ~pos ~len =
     
-    if len < 0 or pos < 0 or pos > String.length s then
+    if len < 0 || pos < 0 || pos > String.length s then
       invalid_arg "Netencoding.QuotedPrintable.decode";
     if pos + len > String.length s then
       invalid_arg "Netencoding.QuotedPrintable.decode";
@@ -664,7 +681,7 @@ module QuotedPrintable = struct
 		  match s.[pos+i+1] with
 		      '\r' ->
 			(* Official soft break *)
-			if i+2 < len & s.[pos+i+2] = '\n' then
+			if i+2 < len && s.[pos+i+2] = '\n' then
 			  count n (i+3)
 			else
 			  count n (i+2)
@@ -776,7 +793,7 @@ module Q = struct
 
   let encode_substring s ~pos ~len =
     
-    if len < 0 or pos < 0 or pos > String.length s then
+    if len < 0 || pos < 0 || pos > String.length s then
       invalid_arg "Netencoding.Q.encode_substring";
     if pos + len > String.length s then
       invalid_arg "Netencoding.Q.encode_substring";
@@ -795,16 +812,12 @@ module Q = struct
     let l = count 0 0 in
     let t = String.create l in
     
-    let hexdigit =
-      [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7';
-	 '8'; '9'; 'A'; 'B'; 'C'; 'D'; 'E'; 'F'; |] in
-
     let k = ref 0 in
 
     let add_quoted c =
       t.[ !k ]   <- '=';
-      t.[ !k+1 ] <- hexdigit.( Char.code c lsr 4 );
-      t.[ !k+2 ] <- hexdigit.( Char.code c land 15 )
+      t.[ !k+1 ] <- hexdigit_uc.( Char.code c lsr 4 );
+      t.[ !k+2 ] <- hexdigit_uc.( Char.code c land 15 )
     in
 
     for i = 0 to len - 1 do
@@ -828,7 +841,7 @@ module Q = struct
 
   let decode_substring s ~pos ~len =
     
-    if len < 0 or pos < 0 or pos > String.length s then
+    if len < 0 || pos < 0 || pos > String.length s then
       invalid_arg "Netencoding.Q.decode_substring";
     if pos + len > String.length s then
       invalid_arg "Netencoding.Q.decode_substring";
@@ -1012,173 +1025,6 @@ module Url = struct
       | _ -> parse_after_amp toklist
   ;;
 
-(* Alternative version. It is commented out because it causes problems
-   in Neturl.
- *)
-
-(*
-
-  (* The Ocamlnet3 version is from Christophe Troestler. 
-     See netcgi.mli for the full copyright message.
-
-     This module is a drop-in replacement of [Netencoding.Url].  Not
-     only it is much faster (about 10 times) but it also avoids the
-     dependency on PCRE (less dependencies are good, especially on
-     windows or if you need to install by hand).
-   *)
-
-  (* Decoding -------------------------------------------------- *)
-  exception Hex_of_char
-
-  let hex_of_char =
-    let code_a = Char.code 'a' - 10
-    and code_A = Char.code 'A' - 10 in
-    function
-    | '0' .. '9' as c -> Char.code c - Char.code '0'
-    | 'a' .. 'f' as c -> Char.code c - code_a
-    | 'A' .. 'F' as c -> Char.code c - code_A
-    | _ -> raise Hex_of_char
-
-  (* Overwrite the part of the range [s.[i0 .. up-1]] with the decoded
-     string.  Returns [i] such that [s.[i0 .. i]] is the decoded
-     string.  Invalid '%XX' are left unchanged.  *)
-  let rec decode_range_loop plus i0 i up s =
-    if i0 >= up then i else begin
-      match String.unsafe_get s i0 with
-      | '+' ->
-	  String.unsafe_set s i plus;
-	  decode_range_loop plus (succ i0) (succ i) up s
-      | '%' when i0 + 2 < up ->
-          let i1 = succ i0 in
-          let i2 = succ i1 in
-          let i0_next =
-            try
-              let v = hex_of_char(String.unsafe_get s i1) lsl 4
-                + hex_of_char(String.unsafe_get s i2) in
-	      String.unsafe_set s i (Char.chr v);
-	      succ i2
-            with Hex_of_char ->
-	      String.unsafe_set s i '%';
-	      i1 in
-	  decode_range_loop plus i0_next (succ i) up s
-      | c ->
-	  String.unsafe_set s i c;
-	  decode_range_loop plus (succ i0) (succ i) up s
-    end
-
-  (* We do not strip heading and trailing spaces of key-value data
-     because it does not conform the specs.  However certain browsers
-     do it, so the user should not rely on them.  See e.g.
-     https://bugzilla.mozilla.org/show_bug.cgi?id=114997#c6 *)
-
-  let decode ?(plus=true) ?(pos=0) ?len s =
-    let real_len = 
-      match len with 
-	| None -> String.length s - pos
-	| Some l -> l in
-    let s = String.sub s pos real_len in
-    let up = decode_range_loop (if plus then ' ' else '+') 0 0 real_len s in
-    if up <> real_len then String.sub s 0 up else s
-
-
-  (* Query parsing -------------------------------------------------- *)
-
-  (* It is ASSUMED that the range is valid i.e., [0 <= low] and [up <=
-     String.length s].  *)
-  let decode_range s low up =
-    if low >= up then "" else
-      let len = up - low in
-      let s = String.sub s low len in
-      let up = decode_range_loop ' ' 0 0 len s in
-      if up <> len then String.sub s 0 up else s
-
-  (* Split the query string [qs] into a list of pairs (key,value).
-     [i0] is the initial index of the key or value, [i] the current
-     index and [up-1] the last index to scan. *)
-  let rec get_key qs i0 i up =
-    if i >= up then [(decode_range qs i0 up, "")] else
-      match String.unsafe_get qs i with
-      | '=' -> get_val qs (i+1) (i+1) up (decode_range qs i0 i)
-      | '&' ->
-	  (* key but no val *)
-	  (decode_range qs i0 i, "") :: get_key qs (i+1) (i+1) up
-      | _ ->
-	  get_key qs i0 (i+1) up
-  and get_val qs i0 i up key =
-    if i >= up then [(key, decode_range qs i0 up)] else
-      match String.unsafe_get qs i with
-      | '&' -> (key, decode_range qs i0 i) :: get_key qs (i+1) (i+1) up
-      | _ -> get_val qs i0 (i+1) up key
-
-
-  let dest_url_encoded_parameters qs =
-    if qs = "" then [] else get_key qs 0 0 (String.length qs)
-
-
-
-  (* Encoding -------------------------------------------------- *)
-
-  let hex = [| '0'; '1'; '2'; '3'; '4'; '5'; '6'; '7'; '8'; '9';
-	       'A'; 'B'; 'C'; 'D'; 'E'; 'F' |]
-  let char_of_hex i = Array.(*unsafe_*)get hex i
-
-  let encode_wrt is_special s0 =
-    let len = String.length s0 in
-    let encoded_length = ref len in
-    for i = 0 to len - 1 do
-      if is_special(String.unsafe_get s0 i) then
-	encoded_length := !encoded_length + 2
-    done;
-    let s = String.create !encoded_length in
-    let rec do_enc i0 i = (* copy the encoded string in s *)
-      if i0 < len then begin
-	let s0i0 = String.unsafe_get s0 i0 in
-	(* It is important to check first that [s0i0] is special in
-	   case [' '] is considered as such a character. *)
-	if is_special s0i0 then begin
-          let c = Char.code s0i0 in
-          let i1 = succ i in
-          let i2 = succ i1 in
-          String.unsafe_set s i '%';
-          String.unsafe_set s i1 (char_of_hex (c lsr 4));
-          String.unsafe_set s i2 (char_of_hex (c land 0x0F));
-          do_enc (succ i0) (succ i2)
-	end
-	else if s0i0 = ' ' then begin
-	  String.unsafe_set s i '+';
-          do_enc (succ i0) (succ i)
-	end
-	else begin
-          String.unsafe_set s i s0i0;
-          do_enc (succ i0) (succ i)
-	end
-      end in
-    do_enc 0 0;
-    s
-
-
-  (* Unreserved characters consist of all alphanumeric chars and the
-     following limited set of punctuation marks and symbols: '-' | '_' |
-     '.' | '!' | '~' | '*' | '\'' | '(' | ')'.  According to RFC 2396,
-     they should not be escaped unless the context requires it. *)
-  let special_rfc2396 = function
-    | ';' | '/' | '?' | ':' | '@' | '&' | '=' | '+' | '$' | ',' (* Reserved *)
-    | '\000' .. '\031' | '\127' .. '\255' (* Control chars and non-ASCII *)
-    | '<' | '>' | '#' | '%' | '"'         (* delimiters *)
-    | '{' | '}' | '|' | '\\' | '^' | '[' | ']' | '`' (* unwise *)
-    | '~' (* causes sometimes problems *)
-	-> true
-    | _ -> false
-  (* ' ' must also be encoded but its encoding '+' takes a single char. *)
-
-  let encode ?(plus=true) s =
-    let is_special =
-      if plus then special_rfc2396
-      else (fun c -> special_rfc2396 c || c = ' ') in
-    encode_wrt is_special s
-
- *)
-
   let mk_url_encoded_parameters params =
     String.concat "&"
       (List.map (fun (name, value) -> encode name ^ "=" ^ encode value) params)
@@ -1470,7 +1316,7 @@ module Html = struct
 
   let rev_etable =
     (* Only code points 0 to 255: *)
-    let a = Array.create 256 "" in
+    let a = Array.make 256 "" in
     List.iter (fun (name,value) -> 
 		 if value <= 255 then
 		   a.(value) <- "&" ^ name ^ ";"
