@@ -1,4 +1,4 @@
-(* $Id: netfs.ml 1661 2011-08-28 22:45:55Z gerd $ *)
+(* $Id: netfs.ml 1809 2012-11-05 23:09:14Z gerd $ *)
 
 type read_flag =
     [ `Skip of int64 | `Binary | `Streaming | `Dummy ]
@@ -630,7 +630,7 @@ let copy ?(replace=false) ?(streaming=false)
     else
       raise(Unix.Unix_error(Unix.ENOSYS,"",""))
   with
-    | Unix.Unix_error(Unix.ENOSYS,_,_) ->
+    | Unix.Unix_error(Unix.ENOSYS,_,_) | Unix.Unix_error(Unix.EXDEV,_,_) ->
 	copy_prim ~streaming orig_fs orig_name dest_fs dest_name
 
 type file_kind = [ `Regular | `Directory | `Symlink | `Other | `None ]
