@@ -1,4 +1,4 @@
-(* $Id: netfs.mli 1661 2011-08-28 22:45:55Z gerd $ *)
+(* $Id: netfs.mli 2195 2015-01-01 12:23:39Z gerd $ *)
 
 (** Class type [stream_fs] for filesystems with stream access to files *)
 
@@ -404,14 +404,15 @@ val local_fs : ?encoding:Netconversion.encoding -> ?root:string ->
 
 (** List:
 
-    - {!Http_fs} allows one to access HTTP-based filesystems
-    - {!Ftp_fs} allows on to access filesystems via FTP
+    - {!Nethttp_fs} allows one to access HTTP-based filesystems
+    - {!Netftp_fs} allows on to access filesystems via FTP
     - {!Shell_fs} allows one to access filesystems by executing shell
       commands. This works locally and via ssh.
 
     There are even some implementations outside Ocamlnet:
-    - {{:http://oss.wink.com/webdav/} Webdav} provides an extension of
-      {!Http_fs} for the full WebDAV set of filesystem operations
+    - {{:http://projects.camlcity.org/projects/webdav.html} Webdav}
+      provides an extension of
+      {!Nethttp_fs} for the full WebDAV set of filesystem operations
 
  *)
 
