@@ -1,4 +1,4 @@
-(* $Id: nethtml.mli 1561 2011-03-07 16:09:25Z gerd $
+(* $Id: nethtml.mli 2244 2015-06-21 13:07:46Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -187,6 +187,7 @@ val parse_document : ?dtd:simplified_dtd ->            (* default: html40_dtd *)
                      ?return_declarations:bool ->      (* default: false *)
                      ?return_pis:bool ->               (* default: false *)
                      ?return_comments:bool ->          (* default: false *)
+                     ?case_sensitive:bool ->
                      Lexing.lexbuf ->
                        document list
   (** Parses the HTML document from a [lexbuf] and returns it. 
@@ -203,12 +204,17 @@ val parse_document : ?dtd:simplified_dtd ->            (* default: html40_dtd *)
    * @param return_comments if set, the parser returns [<!--] .... [-->] comments
    *   as [Element("--",["contents",c],[])] nodes, where [c] is the string inside
    *   [<!--] and [-->]. - By default, comments are skipped.
+   * @param case_sensitive if set, the case of attributes and elements is
+   *   preserved, and when comparing these entities case matters. By default,
+   *   the case does not matter, and attributes and elements are returned with
+   *   lowercased names.
    *)
 
 val parse : ?dtd:simplified_dtd ->            (* default: html40_dtd *)
             ?return_declarations:bool ->      (* default: false *)
             ?return_pis:bool ->               (* default: false *)
             ?return_comments:bool ->          (* default: false *)
+            ?case_sensitive:bool ->
               Netchannels.in_obj_channel ->
                 document list
   (** Parses the HTML document from an object channel and returns it.
