@@ -1,4 +1,4 @@
-(* $Id: netstring_top.ml 1003 2006-09-24 15:17:15Z gerd $
+(* $Id: netstring_top.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -12,7 +12,7 @@ let exec s =
 
 (* Install the printers: *)
 
-exec "#install_printer Mimestring.print_s_param;;";;
+exec "#install_printer Netmime_string.print_s_param;;";;
 exec "#install_printer Neturl.print_url;;";;
 exec "#install_printer Netbuffer.print_buffer;;";;
 exec "#install_printer Netstream.print_in_obj_stream;;";;
