@@ -1,4 +1,4 @@
-(* $Id: neturl.mli,v 2.3 2004/07/18 16:36:24 stolpmann Exp $
+(* $Id: neturl.mli 1662 2011-08-29 23:05:06Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -10,7 +10,8 @@
  * {b Contents}
  *
  * - {!Neturl.interface}
- * - {!Neturl.tutorial}
+ *
+ * The tutorial has been moved to {!Neturl_tut}.
  *)
 
 (** {1:interface Interface}
@@ -58,12 +59,12 @@
  * Note that nothing is said about the character set/encoding of URLs.
  * Some protocols and standards prefer UTF-8 as fundamental encoding
  * and apply the [%]-encoding on top of it; i.e. the byte sequence
- * representing a character in UTF-8 is [%]-encoded. 
+ * representing a character in UTF-8 is [%]-encoded.
  *
  * {b Standards Compliance}
  *
  * This module implements RFC 1738 and RFC 1808. There is also a newer
- * RFC, 2396, updating the former RFCs, but this module is not fully 
+ * RFC, 2396, updating the former RFCs, but this module is not fully
  * compatible with RFC 2396. The following (minor) problems may occur:
  *
  * - The module escapes more characters than needed. All characters that
@@ -80,6 +81,9 @@
  * - Authorities may be terminated by a question mark, as in
  *   ["http://host?query"]. This is illegal in RFC 1738. The consequence
  *   is, however, that question marks in user strings must be escaped.
+ *
+ * RFC 3986 introduces IPv6 addresses. These are now supported (but see
+ * the comments below).
  *)
 
 exception Malformed_URL
@@ -88,8 +92,8 @@ exception Malformed_URL
  *)
 
 val extract_url_scheme : string -> string
-  (** Returns the URL scheme from the string representation of an URL. 
-   * E.g. [extract_url_scheme "http://host/path" = "http"]. 
+  (** Returns the URL scheme from the string representation of an URL.
+   * E.g. [extract_url_scheme "http://host/path" = "http"].
    * The scheme name is always converted to lowercase characters.
    * Raises [Malformed_URL] if the scheme name is not found.
    *)
@@ -123,7 +127,7 @@ type url_syntax =
  *
  * The function [url_is_valid] is applied when a fresh URL is created
  * and must return [true]. This function allows it to add an arbitrary
- * validity criterion to [url_syntax]. (Note that the URL passed to 
+ * validity criterion to [url_syntax]. (Note that the URL passed to
  * this function is not fully working; you can safely assume that the
  * accessor functions [url_scheme] etc. can be applied to it.)
  *
@@ -191,7 +195,7 @@ val common_url_syntax : (string, url_syntax) Hashtbl.t
    * - ["file"]: scheme, host?, path
    * - ["ftp"]: scheme, user?, password?, host, port?, path?, param?
    *   Note: param is not checked.
-   * - ["http"], ["https"]: 
+   * - ["http"], ["https"]:
    *   scheme, user?, password?, host, port?, path?, query?
    * - ["mailto"]: scheme, other, query? (RFC 2368)
    * - ["pop"], ["pops"]: scheme, user?, user_param?, password?, host, port?
@@ -203,14 +207,14 @@ val common_url_syntax : (string, url_syntax) Hashtbl.t
    *   relative URLs as described in the RFC. When analysing this kind of URL,
    *   it is recommended to re-parse it with "param" enabled.
    * - ["news"]: scheme, other (RFC 1738)
-   * - ["nntp"], ["nntps"]: scheme, host, port?, path (with two components) 
+   * - ["nntp"], ["nntps"]: scheme, host, port?, path (with two components)
    *   (RFC 1738)
    * - ["data"]: scheme, other (RFC 2397). "other" is not further decomposed.
    * - ["ipp"], ["ipps"]: scheme, host, port? , path?, query? (RFC 3510)
    * - ["cid"], ["mid"]: Content/message identifiers: scheme, other
    *
    * Notes:
-   * - These syntax descriptions can be weakened for partial/relative URLs 
+   * - These syntax descriptions can be weakened for partial/relative URLs
    *   by changing the required parts to optional parts: See the function
    *   [partial_url_syntax].
    * - None of the descriptions allows fragments. These can be enabled by
@@ -236,7 +240,9 @@ val make_url :
       ?user_param:string list ->
       ?password:string ->
       ?host:string ->
+      ?addr:Unix.inet_addr ->
       ?port:int ->
+      ?socksymbol: Netsockaddr.socksymbol ->
       ?path:string list ->
       ?param:string list ->
       ?query:string ->
@@ -245,14 +251,19 @@ val make_url :
       url_syntax ->
       url
   (** Creates a URL from components:
-   * 
+   *
    * - The components [scheme] and [host] are simple strings to which the
-   *   [%]-encoding is not applicable.
+   *   [%]-encoding is not applicable. [host] may be a (DNS) name, an
+   *   IPv4 address as "dotted quad", or an IPv6 address enclosed in
+   *   brackets.
+   * - [addr] also sets [host], but directly from an [inet_addr].
    * - The component [port] is a simple number. Of course, the [%]-encoding
    *   is not applicable, too.
+   * - [socksymbol] sets both [host] and [port] from the socksymbol of
+   *   type [`Inet] or [`Inet_byname].
    * - The components [user], [password], [query], [fragment], and [other]
    *   are strings which may contain [%]-encoded characters. By default,
-   *   you can pass any string for these components, and problematic characters 
+   *   you can pass any string for these components, and problematic characters
    *   are automatically encoded. If you set [encoded:true], the passed
    *   strings must already be encoded, but the function checks whether
    *   the encoding is syntactically correct.
@@ -267,8 +278,11 @@ val make_url :
    *   means that they are not part of the constructed URL.
    *   See below for the respresentation of these components.
    *
+   * [socksymbol] has precedence over [addr], which has precedence over
+   * [host]. [socksymbol] also has precedence over [port].
+   *
    * The strings representing the components do not
-   * contain the characters separating the components from each other. 
+   * contain the characters separating the components from each other.
    *
    * The created URL must conform to the [url_syntax], i.e.:
    * - The URL must only contain components which are recognized by the
@@ -279,21 +293,21 @@ val make_url :
    *
    * The path of a URL is represented as a list of ['/']-separated path
    * components. i.e.
-   * 
-   * [ [ s1; s2; ...; sN ] ]  represents the path  
+   *
+   * [ [ s1; s2; ...; sN ] ]  represents the path
    *                        [s1 ^ "/" ^ s2 ^ "/" ^ ... ^ "/" ^ sN]
    *
    * As special cases:
    * -  [[]]                   is the non-existing path
    * -  [[ "" ]]               is ["/"]
    * -  [[ "";"" ]]            is illegal
-   * 
+   *
    * Except of [s1] and [sN], the path components must not be empty strings.
    *
    * To avoid ambiguities, it is illegal to create URLs with both relative
    * paths ([s1 <> ""]) and host components.
    *
-   * Parameters of URLs ([param] and [user_param]) are components 
+   * Parameters of URLs ([param] and [user_param]) are components
    * beginning with [';']. The list
    * of parameters is represented as list of strings where the strings
    * contain the value following [';'].
@@ -307,7 +321,9 @@ val modify_url :
       ?user_param:string list ->
       ?password:string ->
       ?host:string ->
+      ?addr:Unix.inet_addr ->
       ?port:int ->
+      ?socksymbol: Netsockaddr.socksymbol ->
       ?path:string list ->
       ?param:string list ->
       ?query:string ->
@@ -315,7 +331,7 @@ val modify_url :
       ?other:string ->
       url ->
       url
-  (** Modifies the passed components and returns the modified URL. 
+  (** Modifies the passed components and returns the modified URL.
    * The modfied URL shares unmodified components with the original
    * URL.
    *)
@@ -341,7 +357,7 @@ val remove_from_url :
    *)
 
 val default_url :
-      ?encoded:bool -> 
+      ?encoded:bool ->
       ?scheme:string ->
       ?user:string ->
       ?user_param:string list ->
@@ -390,9 +406,9 @@ val url_of_string : url_syntax -> string -> url
 val string_of_url : url -> string
   (** Returns the URL as string *)
 
-val parse_url : 
+val parse_url :
       ?schemes:(string, url_syntax) Hashtbl.t ->
-      ?base_syntax:url_syntax -> 
+      ?base_syntax:url_syntax ->
       ?accept_8bits:bool ->
       ?enable_fragment:bool ->
       string -> url
@@ -418,6 +434,18 @@ val parse_url :
    *   are always recognized.
    *)
 
+val fixup_url_string : ?escape_hash:bool -> string -> string
+  (** Escapes some unsafe or "unwise" characters that are commonly used
+    * in URL strings: space, < > \{ \} ^ \\ | and double quotes.
+    * Call this function before parsing the URL to support these
+    * characters.
+    *
+    * If [escape_hash] is set, '#' is also escaped.
+    * 
+    * Change: Since Ocamlnet-3.4, square brackets are no longer fixed up,
+    * because they have now a legal use to denote IPv6 addresses.
+   *)
+
 val url_provides :
       ?scheme:bool ->
       ?user:bool ->
@@ -451,16 +479,35 @@ val url_other     : ?encoded:bool -> url -> string
    * unless [encoded:true] is set.
    * If the component does not exist, the exception [Not_found]
    * is raised.
+   *
+   * Note that IPv6 addresses, when returned by [url_host], are enclosed
+   * in square brackets. Modules calling [url_host] may require porting
+   * to support this syntax variant.
+   *)
+
+val url_addr : url -> Unix.inet_addr
+  (** If the [host] part of the URL is an IP address, the address is returned.
+      Works for IPv4 and IPv6 addresses. Otherwise [Not_found] is raised.
+   *)
+
+val url_socksymbol : url -> int -> Netsockaddr.socksymbol
+  (** [url_socksymbol url default_port]: Returns the [host] and [port] parts
+      of the URL as [socksymbol]. If the port is missing in the URL,
+      [default_port] is substituted. If the [host] is missing in the URL
+      the exception [Not_found] is raised.
    *)
 
 val split_path : string -> string list
   (** Splits a ['/']-separated path into components (e.g. to set up the
    * [path] argument of [make_url]).
-   * E.g. 
+   * E.g.
    * {[
    * split_path "a/b/c" = [ "a"; "b"; "c" ],
    * split_path "/a/b"  = [ ""; "a"; "b" ],
    * split_path "a/b/"  = [ "a"; "b"; "" ] ]}
+   * Beware that [split_path ".."] returns [[".."]] while [split_path "../"]
+   * returns [[".."; ""]].  The two will behave differently, for example
+   * when used with {!Neturl.apply_relative_url}.
    *)
 
 val join_path : string list -> string
@@ -483,7 +530,7 @@ val norm_path : string list -> string list
    *
    *           means: "a/." = "a/"}
    * {- [norm_path ["a"; "b"; "."] = ["a"; "b"; ""]]
-   * 
+   *
    *           means: "a/b/." = "a/b/"}
    * {- [norm_path ["a"; "."; "b"; "."] = ["a"; "b"; ""]]
    *
@@ -519,16 +566,26 @@ val apply_relative_url : url -> url -> url
    * function implements RFC 1808.
    *
    * It is not necessary that [rel] has the same syntax as [base].
-   * Note, however, that it is checked whether the resulting URL is syntactically
-   * correct with the syntax of [base]. If not, the exception [Malformed_URL] 
-   * will be raised.
+   * Note, however, that it is checked whether the resulting URL is
+   * syntactically correct with the syntax of [base]. If not, the
+   * exception [Malformed_URL] will be raised.
+   *
+   * Examples (the URLs are represented as strings, see {!Neturl.split_path}
+   * to split them for {!Neturl.make_url}):
+   *
+   * base="x/y", url="a/b" => result="x/a/b"
+   * base="x/y/", url="a/b" => result="x/y/a/b"
+   * base="x/y/..", url="a/b" => result="x/y/a/b"   (beware!)
+   * base="x/y/../", url="a/b" => result="x/a/b"
    *)
 
 val ensure_absolute_url : ?base:url -> url -> url
-  (** If the anonymous URL is absolute, it is just returned as result of 
-   * this function. If the URL is relative, it is tried to make it 
-   * absolute by resolving it relative to [base]. If there is no [base],
-   * this will fail, and the function raises [Malformed_URL].
+  (** If the anonymous URL is absolute, it is just returned as result of
+   * this function. If the URL is relative, it is tried to make it
+   * absolute by resolving it relative to [base]. If there is no [base]
+   * or if the the base URL does not allow the parts that would be added
+   * (e.g. if the anonymous URL possesses a fragment and [base] does not
+   * allow that), this will fail, and the function raises [Malformed_URL].
    *)
 
 val file_url_of_local_path : ?getcwd:(unit -> string) -> string -> url
@@ -538,24 +595,24 @@ val file_url_of_local_path : ?getcwd:(unit -> string) -> string -> url
    *
    * Note that no character set conversions are performed.
    *
-   * Win32: The input path name may use forward or backward slashes. 
+   * Win32: The input path name may use forward or backward slashes.
    * Absolute paths with drive letters and UNC paths are recognised.
    * Relative paths with drive letters, however, are not recognised
    * (e.g. ["c:file"]), as it is not possible to access the drive-specific
    * working directory from the O'Caml runtime.
    *
-   * Cygwin: The input path name may use forward or backward slashes. 
+   * Cygwin: The input path name may use forward or backward slashes.
    * Absolute paths with drive letters and UNC paths are recognised.
    * The former are translated to ["/cygdrive"] names.
    *
-   * @param getcwd The function returns the path taken as current working 
+   * @param getcwd The function returns the path taken as current working
    *   directory. Note that for
    *   Win32 this must be either an absolute name with drive letter,
    *   or an UNC path. Default: [Sys.getcwd]
    *)
 
 val local_path_of_file_url : url -> string
-  (** Extracts the path from an absolute file URL, and returns a 
+  (** Extracts the path from an absolute file URL, and returns a
    * correct path name.
    *
    * If the URL is not a file URL, or is not absolute, the function will
@@ -610,7 +667,7 @@ val data_contents : url -> XXX
  *   --> [ ""; "a"; "b"; "" ]          (* sic! *)
  * url_path ~encoded:true u;;
  *   --> [ ""; "a"; "%62"; "" ]
- * let v = make_url 
+ * let v = make_url
  *   ~path:[ ".."; "c" ]
  *   ~fragment:"near-the-#-character"
  *   { (partial_url_syntax http) with url_enable_fragment = Url_part_allowed };;
@@ -625,242 +682,3 @@ val data_contents : url -> XXX
 
 (* ---------------------------------------------------------------------- *)
 
-(** {1:tutorial Tutorial} 
- *
- * This module is a quite flexible parser for various kinds of URLs
- * occuring in practice. The syntax is configurable such that one
- * URL module can handle a lot of URL types in a generic way.
- *
- * {b Generic Parsing}
- *
- * In order to parse an absolute URL (beginning with a scheme identifier 
- * like "http:...") of unknown type just call
- * {[ 
- * let url = parse_url "http://me\@server/directory"
- * ]}
- *
- * By default, this function can parse all URL types listed
- * at {!Neturl.common_url_syntax}. However, the default configuration
- * implies also that
- * - relative URLs cannot be parsed
- * - fragment identifiers are rejected (i.e. the part after the hash
- *   mark like in "http://server/document#location")
- * - characters are rejected when the most significant bit (MSB) is set
- *
- * The latter two features can be simply enabled by passing the
- * arguments [~enable_fragment:true] and [~accept_8bits:true], 
- * respectively.
- *
- * The restriction that relative URLs are rejected has to do with
- * the problem that context information is missing. Because the scheme
- * identifier (like "http") is not available, the function does not
- * know which syntax the relative URL should have.  For example,
- * the relative URL [dir/file?x=1] is differently parsed when
- * it is taken relative to an [http] URL and when it is interpreted
- * relative to an [ftp] URL. In the first case, the path component
- * of the URL is ["dir/file"] and the query component is ["?x=1"],
- * but in the latter case the path component is ["dir/file?x=1"],
- * and a query component is not allowed.
- *
- * The solution is that the syntax of the base URL, relative to which
- * the URL is seen, must be passed as additional argument. Under the
- * assumption that [base_url] is the base URL, use
- * {[
- * let url = parse_url 
- *             ~base_syntax:(url_syntax_of_url base_url) 
- *             "/dir/file?x=1"
- * ]}
- * Of course, this assumes that the base URL is known when the url
- * is parsed.
- *
- * {b Parsing For a Certain Syntax}
- *
- * The function [url_of_string] is also a parser, but you must pass
- * the URL syntax as argument, e.g.
- * {[
- * let url = url_of_string syntax "ipp://server/printer"
- * ]}
- * Pass as [syntax] one of the elements of {!Neturl.common_url_syntax},
- * e.g.
- * {[ let syntax = Hashtbl.find common_url_syntax "ipp" ]}
- * or a self-defined syntax.
- *
- * {b Printing URLs}
- *
- * This is much easier, just call [string_of_url] to convert an URL
- * to a string. It is ensured that every URL always has an
- * unambiguous representation as string.
- *
- * {b URL Components}
- *
- * Internally, the parsed URL is decomposed into its components. This
- * module supports two ways of decomposition:
- * + [scheme://user;userparams:password\@host:port/path;params?query#fragment]
- * + [scheme:other;params?query#fragment]
- *
- * The first form is used for services that directly connect to a
- * certain service running on a certain host. The second form can
- * be used for everything else not falling under this category.
- *
- * Examples:
- * {ul
- * {- [http://me:abrakadabra\@server/dir?x=5#section1]
- *
- *    scheme=["http"], user=["me"], password=["abrakadabra"], host=["server"],
- *    path=["/dir"], query=["x=5"], fragment=["section1"] }
- * {- [pop://you;auth=digest-md5\@mail]
- *
- *    scheme=["pop"], user=["you"], user_params=[["auth=digest-md5"]], host=["mail"] }
- * {- [mailto:gerd\@gerd-stolpmann.de?cc=you\@domain.com]
- * 
- *    scheme=["mailto"], other=["gerd\@gerd-stolpmann.de"],
- *    query=["cc=you\@domain.com"] }
- * }
- *
- * It is important to mention that the decomposition is not fully
- * performed, but only down to a certain level. For example, the
- * query ["x=5"] could be further analysed and be split into
- * the syntactic parts ["x"] and ["5"]. However, this is not done,
- * just because the author seeked a compromise between the depth
- * of analysis and the genericy of application.
- *
- * {b URL Escaping}
- *
- * In order to represent the so-called unsafe characters, one can
- * use [%]-escaping in URLs. For example, this URL contains a 
- * password with [\@], an unsafe character encoded as [%40]:
- * {[ http://user:!$%40?\@server ]}
- * The question is how this module handles such escapings.
- *
- * It is surprising that the URL parser does not decode these
- * escaped forms (it checks, however, whether they are syntactically
- * correct). Internally, the components are stored as parsed,
- * and one can even retrieve them in their original form.
- * The function [url_password] returns the password component.
- * Applied to the above URL, one can get the password in
- * its original, "encoded" form, or as decoded string:
- * - [url_password ~encoded:true url] returns ["!$%40?"]
- * - [url_password url] returns ["!$\@?"]
- *
- * {b Representation of URL Components}
- *
- * The URL components can be retrieved with the functions
- * - [url_scheme]
- * - [url_user]
- * - [url_user_param]
- * - [url_password]
- * - [url_host]
- * - [url_port]
- * - [url_path]
- * - [url_param]
- * - [url_query]
- * - [url_fragment]
- * - [url_other]
- *
- * Most components are just strings. Of course, the port number is an
- * integer.
- *
- * The path component ([url_path]) has a non-obvious representation. The path
- * is represented as string list, e.g. "a/b/c" is represented
- * as [ ["a";"b";"c"] ]. Note, however, that absolute paths have
- * an empty string at the beginning of the list, e.g.
- * "/a/b/" is [ [""; "a"; "b"; "" ] ]. In most cases, the paths
- * found in URLs are absolute, and because of this it is quite
- * common to find this empty string at the beginning of the
- * path list. The corner cases are:
- * - [ [] ] is used when the path is missing in the URL
- * - [ [ "" ] ] is "/"
- * - [ [ ""; "" ] ] is considered as illegal
- *
- * The last two cases are somewhat arbitrary.
- *
- * There is the helper function [split_path] to convert the string
- * representation of paths into the list representation.
- *
- * The parameters ([url_user_param] and [url_param]) are lists, too.
- * A parameter starts with a semicolon as delimiter and runs until
- * the next component, which can be another parameter. The contents,
- * i.e. the values after the semicolons are put into the list. For example,
- * the parameter ";auth=unix;type=i" is represented as
- * [ ["auth=unix"; "type=i"] ].
- *
- * {b Hint: Getting Query Arguments}
- *
- * The query component is represented as a single string. When queries
- * use the standard syntax "name1=value1&name2=value2&...", one can
- * parse this string using
- * {[
- * let args = Netencoding.Url.dest_url_encoded_parameters
- *              (url_query ~encoded:true url)
- * ]}
- * Note that [encoded:true] is needed.
- *
- * {b Creating and Modifying URLs}
- *
- * In order to create a URL for a certain syntax, call [make_url]:
- * {[
- * let url = make_url 
- *             ~scheme:"http"
- *             ~user:"user"
- *             ~password:"!$\@?"
- *             ~host:"server"
- *             syntax
- * ]}
- * It is checked whether the URL conforms to the passed syntax. By default,
- * the components are passed in decoded form, and [make_url] automatically
- * encodes them if necessary (here, for example, the at sign in the
- * password). Alternatively, one can set [~encoded:true], and pass the
- * already escaped components. In this case, [make_url] checks whether
- * the encoding is sufficient to represent the URL as string.
- *
- * The functions [modify_url], [default_url], [undefault_url], and
- * [remove_from_url] can be used to modify an existing URL.
- *
- * {b Relative URLs}
- *
- * A URL is relative when the scheme identifier at the beginning is
- * omitted. In this case, the URL can be transformed to an absolute
- * URL when the base URL is known. The algorithm for this is defined
- * in RFC 1808, and quite complicated. It is implemented in
- * [apply_relative_url], but usually {!Neturl.ensure_absolute_url}
- * is the more convenient function. Just call
- * {[ let url' = ensure_absolute_url ~base url ]}
- * to convert [url] to its absolute counterpart [url'] when it is
- * relative, and to pass the URL unchanged when it is already
- * absolute.
- *)
-
-
-
-(* ======================================================================
- * History:
- * 
- * $Log: neturl.mli,v $
- * Revision 2.3  2004/07/18 16:36:24  stolpmann
- * 	New: Neturl.file_url_of_local_path, local_path_of_file_url.
- * 	URL updates according to RFC 2396.
- *
- * Revision 2.2  2004/07/12 22:57:01  stolpmann
- * 	Added: parse_url, ensure_absolute_url
- * 	Support for userparam.
- * 	Syntax can prevent relative URLs.
- * 	A lot of new syntaxes (pop, imap, ...)
- *
- * Revision 2.1  2001/09/14 14:22:34  stolpmann
- * 	Initial revision (sourceforge)
- *
- *
- * ======================================================================
- * Revision 1.3  2000/06/26 22:57:49  gerd
- * 	Change: The record 'url_syntax' has an additional component
- * 'url_accepts_8bits'. Setting this option to 'true' causes that
- * the bytes >= 0x80 are no longer rejected.
- *
- * Revision 1.2  2000/06/25 22:55:47  gerd
- * 	Doc update.
- *
- * Revision 1.1  2000/06/24 20:19:59  gerd
- * 	Initial revision.
- *
- * 
- *)
