@@ -1,4 +1,4 @@
-(* $Id: netsys.ml 1711 2012-02-16 16:24:35Z gerd $ *)
+(* $Id: netsys.ml 2195 2015-01-01 12:23:39Z gerd $ *)
 
 open Printf
 
@@ -47,6 +47,29 @@ let is_absolute path =
   else
     path <> "" && path.[0] = '/'
 
+let abspath_w32 path =
+  (* full path: resolves relative paths, and eliminates . and ..
+     long path: gets away with 8.3 paths, and converts file name case
+   *)
+  Netsys_win32.get_long_path_name
+    (Netsys_win32.get_full_path_name path)
+
+
+let abspath path =
+  if is_win32 then
+    abspath_w32 path
+  else
+    try
+      Netsys_posix.realpath path
+    with
+      | Invalid_argument _ ->
+           (* this is sub-standard, let's hope we never run into this *)
+           if is_absolute path then
+             path
+           else
+             Filename.concat
+               (Unix.getcwd())
+               path
 
 let restart = Netsys_impl_util.restart
 let restart_tmo = Netsys_impl_util.restart_tmo
@@ -77,6 +100,103 @@ let getpeername fd =
 	raise(Unix.Unix_error(Unix.ENOTCONN,a1,a2))
 
 
+let domain_of_inet_addr addr =
+  Unix.domain_of_sockaddr(Unix.ADDR_INET(addr,0))
+
+let protostring_of_inet_addr ip = (Obj.magic ip)
+
+let inet_addr_of_protostring s =
+  let l = String.length s in
+  if l = 4 || l = 16 then (Obj.magic s) else
+    invalid_arg "Netsys.inet_addr_of_protostring"
+
+external _exit : int -> unit = "netsys__exit";;
+(* same external also in netsys_signal.ml *)
+
+let binop_inet_addr f (ip1 : Unix.inet_addr) (ip2 : Unix.inet_addr) =
+  let s1 = (Obj.magic ip1 : string) in
+  let s2 = (Obj.magic ip2 : string) in
+  let l = String.length s1 in
+  if l <> String.length s2 then
+    failwith "logand_inet_addr";
+  let s3 = String.create l in
+  for k = 0 to l-1 do
+    s3.[k] <- Char.chr(f (Char.code s1.[k]) (Char.code s2.[k]));
+  done;
+  (Obj.magic s3 : Unix.inet_addr)
+
+let logand_inet_addr =
+  binop_inet_addr ( land )
+
+let logor_inet_addr =
+  binop_inet_addr ( lor )
+
+let logxor_inet_addr =
+  binop_inet_addr ( lxor )
+
+let lognot_inet_addr ip =
+  binop_inet_addr
+    (fun p1 p2 -> lnot p1)
+    ip
+    ip
+
+let norm_inet_addr (ip : Unix.inet_addr) =
+  if String.length (Obj.magic ip) = 16 then
+    let ip1 =
+      logand_inet_addr
+        ip
+        (Unix.inet_addr_of_string "ffff:ffff:ffff:ffff:ffff:ffff::0") in
+    if (ip1 = (Unix.inet_addr_of_string "0000:0000:0000:0000:0000:ffff::0") ||
+        ip1 = (Unix.inet_addr_of_string "0000:0000:0000:0000:0000:0000::0")) &&
+       ip <> Unix.inet6_addr_any && ip <> Unix.inet6_addr_loopback
+    then
+      Obj.magic (String.sub (Obj.magic ip) 12 4)
+    else
+      ip
+  else
+    ip
+
+let ipv6_inet_addr (ip : Unix.inet_addr) =
+  if String.length (Obj.magic ip) = 4 then
+    Obj.magic (String.make 10 '\x00' ^ String.make 2 '\xff' ^ Obj.magic ip)
+  else
+    ip
+
+
+let is_ipv4_inet_addr (ip : Unix.inet_addr) =
+  String.length (Obj.magic (norm_inet_addr ip)) = 4
+
+let is_ipv6_inet_addr (ip : Unix.inet_addr) =
+  String.length (Obj.magic (norm_inet_addr ip)) = 16
+
+let is_multicast_inet_addr ip =
+  let ip1 = norm_inet_addr ip in
+  if String.length (Obj.magic ip1) = 4 then
+    logand_inet_addr
+      ip1
+      (Unix.inet_addr_of_string "240.0.0.0")
+    = (Unix.inet_addr_of_string "224.0.0.0")
+  else
+    if String.length (Obj.magic ip1) = 16 then
+      logand_inet_addr
+        ip1
+        (Unix.inet_addr_of_string "ffff::0")
+      = (Unix.inet_addr_of_string "ff00::0")
+    else
+      false
+
+
+external test_for_ip6_global_addr : unit -> bool
+  = "netsys_test_for_ip6_global_addr"
+
+let ipv6 = ref (test_for_ip6_global_addr())
+
+let is_ipv6_system() = !ipv6
+
+let set_ipv6_system b = ipv6 := b
+  
+
+
 type fd_style =
     [ `Read_write
     | `Recv_send of Unix.sockaddr * Unix.sockaddr
@@ -88,6 +208,7 @@ type fd_style =
     | `W32_process
     | `W32_input_thread
     | `W32_output_thread
+    | `TLS of Netsys_crypto_types.file_tls_endpoint
     ]
 
 
@@ -145,10 +266,12 @@ let get_fd_style fd =
 		("get_fd_style: Exception: " ^ Netexn.to_string e);
 	      assert false
 
-let string_of_sockaddr =
+let string_of_sockaddr ?(norm=false) =
   function
     | Unix.ADDR_INET(inet,port) as addr ->
-	( match Unix.domain_of_sockaddr addr with
+        let inet =
+          if norm then norm_inet_addr inet else inet in
+	( match domain_of_inet_addr inet with
 	    | Unix.PF_INET ->
 		Unix.string_of_inet_addr inet ^ ":" ^ string_of_int port
 	    | Unix.PF_INET6 ->
@@ -173,6 +296,7 @@ let string_of_fd_style =
     | `W32_process -> "W32_process" 
     | `W32_input_thread -> "W32_input_thread"
     | `W32_output_thread -> "W32_output_thread"
+    | `TLS _ -> "TLS"
 
 let string_of_fd fd =
   let st = get_fd_style fd in
@@ -231,6 +355,14 @@ let wait_until_readable fd_style fd tmo =
       | `W32_process
       | `W32_output_thread ->
 	  sleep tmo; false (* never *)
+      | `TLS ep ->
+          let module Endpoint = 
+            (val ep : Netsys_crypto_types.FILE_TLS_ENDPOINT) in
+          Endpoint.TLS.recv_will_not_block Endpoint.endpoint || (
+	    let l,_,_ = 
+              restart_tmo (Unix.select [Endpoint.rd_file] [] []) tmo in
+	    l <> []
+          )
       | _ ->
 	  let l,_,_ = restart_tmo (Unix.select [fd] [] []) tmo in
 	  l <> []
@@ -261,6 +393,11 @@ let wait_until_writable fd_style fd tmo =
       | `W32_input_thread 
       | `W32_process ->
 	  sleep tmo; false (* never *)
+      | `TLS ep ->
+          let module Endpoint = 
+            (val ep : Netsys_crypto_types.FILE_TLS_ENDPOINT) in
+	  let l,_,_ = restart_tmo (Unix.select [] [Endpoint.wr_file] []) tmo in
+	  l <> []
       | _ ->
 	  let _,l,_ = restart_tmo (Unix.select [] [fd] []) tmo in
 	  l <> []
@@ -297,6 +434,30 @@ let is_writable fd_style fd = wait_until_writable fd_style fd 0.0
 let is_prird fd_style fd = wait_until_prird fd_style fd 0.0
 
 
+let rec restart_wait mode fd_style fd f arg =
+  try
+    f arg
+  with
+    | Unix.Unix_error(Unix.EINTR,_,_) ->
+	restart_wait mode fd_style fd f arg
+    | Unix.Unix_error(Unix.EAGAIN,_,_)
+    | Unix.Unix_error(Unix.EWOULDBLOCK,_,_) ->
+        (match mode with
+           | `R -> 
+                ignore(wait_until_readable fd_style fd (-1.0));
+                restart_wait mode fd_style fd f arg
+           | `W -> 
+                ignore(wait_until_writable fd_style fd (-1.0));
+                restart_wait mode fd_style fd f arg
+        )
+    | Netsys_types.EAGAIN_RD ->
+         ignore(wait_until_readable fd_style fd (-1.0));
+         restart_wait mode fd_style fd f arg
+    | Netsys_types.EAGAIN_WR ->
+         ignore(wait_until_writable fd_style fd (-1.0));
+         restart_wait mode fd_style fd f arg
+
+
 let gwrite fd_style fd s pos len =
   dlogr (fun () -> sprintf "gwrite fd=%Ld len=%d"
 	   (int64_of_file_descr fd) len);
@@ -322,6 +483,8 @@ let gwrite fd_style fd s pos len =
     | `W32_output_thread ->
 	let othr = Netsys_win32.lookup_output_thread fd in
 	Netsys_win32.output_thread_write othr s pos len
+    | `TLS endpoint ->
+        Netsys_tls.send (Netsys_tls.endpoint endpoint) s pos len
 
 
 let rec really_gwrite fd_style fd s pos len =
@@ -332,9 +495,13 @@ let rec really_gwrite fd_style fd s pos len =
   with
     | Unix.Unix_error(Unix.EINTR, _, _) ->
 	really_gwrite fd_style fd s pos len
-    | Unix.Unix_error( (Unix.EAGAIN | Unix.EWOULDBLOCK), _, _) ->
+    | Unix.Unix_error( (Unix.EAGAIN | Unix.EWOULDBLOCK), _, _)
+    | Netsys_types.EAGAIN_WR ->
 	ignore(wait_until_writable fd_style fd (-1.0));
 	really_gwrite fd_style fd s pos len
+    | Netsys_types.EAGAIN_RD ->
+	ignore(wait_until_readable fd_style fd (-1.0));
+	really_gwrite fd_style fd s pos len
 
 
 let gread fd_style fd s pos len =
@@ -362,6 +529,8 @@ let gread fd_style fd s pos len =
     | `W32_input_thread ->
 	let ithr = Netsys_win32.lookup_input_thread fd in
 	Netsys_win32.input_thread_read ithr s pos len
+    | `TLS endpoint ->
+        Netsys_tls.recv (Netsys_tls.endpoint endpoint) s pos len
 
 let blocking_gread fd_style fd s pos len =
   let rec loop pos len p =
@@ -375,9 +544,13 @@ let blocking_gread fd_style fd s pos len =
       with
 	| Unix.Unix_error(Unix.EINTR, _, _) ->
 	    loop pos len p
-	| Unix.Unix_error( (Unix.EAGAIN | Unix.EWOULDBLOCK), _, _) ->
+	| Unix.Unix_error( (Unix.EAGAIN | Unix.EWOULDBLOCK), _, _)
+        | Netsys_types.EAGAIN_RD ->
 	    ignore(wait_until_readable fd_style fd (-1.0));
 	    loop pos len p
+        | Netsys_types.EAGAIN_WR ->
+	    ignore(wait_until_writable fd_style fd (-1.0));
+	    loop pos len p
     else
       p
   in
@@ -479,6 +652,8 @@ let gshutdown fd_style fd cmd =
 	  let othr = Netsys_win32.lookup_output_thread fd in
 	  Netsys_win32.close_output_thread othr
 	)
+    | `TLS endpoint ->
+        Netsys_tls.shutdown (Netsys_tls.endpoint endpoint) cmd
     | _ ->
 	raise Shutdown_not_supported
 
@@ -569,7 +744,21 @@ let gclose fd_style fd =
 	  "Unix.close" fd_detail
 	  Unix.close fd;
 	Netsys_win32.unregister fd
-
+    | `TLS endpoint ->
+        Netsys_tls.shutdown (Netsys_tls.endpoint endpoint) Unix.SHUTDOWN_ALL;
+        let module Endpoint =
+          (val endpoint : Netsys_crypto_types.FILE_TLS_ENDPOINT) in
+	catch_exn
+	  "Unix.close" fd_detail
+	  Unix.close Endpoint.rd_file;
+        if Endpoint.wr_file <> Endpoint.rd_file then
+	  catch_exn
+	    "Unix.close" fd_detail
+	    Unix.close Endpoint.wr_file;
+        if fd <> Endpoint.rd_file then
+	  catch_exn
+	    "Unix.close" fd_detail
+	    Unix.close fd
 
 
 external unix_error_of_code : int -> Unix.error = "netsys_unix_error_of_code"
@@ -606,70 +795,6 @@ let connect_check fd =
 				"connect_check", detail))
   )
 
-(* Misc *)
-
-let domain_of_inet_addr addr =
-  Unix.domain_of_sockaddr(Unix.ADDR_INET(addr,0))
-
-let protostring_of_inet_addr ip = (Obj.magic ip)
-
-let inet_addr_of_protostring s =
-  let l = String.length s in
-  if l = 4 || l = 16 then (Obj.magic s) else
-    invalid_arg "Netsys.inet_addr_of_protostring"
-
-external _exit : int -> unit = "netsys__exit";;
-(* same external also in netsys_signal.ml *)
-
-let binop_inet_addr f (ip1 : Unix.inet_addr) (ip2 : Unix.inet_addr) =
-  let s1 = (Obj.magic ip1 : string) in
-  let s2 = (Obj.magic ip2 : string) in
-  let l = String.length s1 in
-  if l <> String.length s2 then
-    failwith "logand_inet_addr";
-  let s3 = String.create l in
-  for k = 0 to l-1 do
-    s3.[k] <- Char.chr(f (Char.code s1.[k]) (Char.code s2.[k]));
-  done;
-  (Obj.magic s3 : Unix.inet_addr)
-
-let logand_inet_addr =
-  binop_inet_addr ( land )
-
-let logor_inet_addr =
-  binop_inet_addr ( lor )
-
-let logxor_inet_addr =
-  binop_inet_addr ( lxor )
-
-let lognot_inet_addr ip =
-  binop_inet_addr
-    (fun p1 p2 -> lnot p1)
-    ip
-    ip
-
-let is_ipv4_inet_addr (ip : Unix.inet_addr) =
-  String.length (Obj.magic ip) = 4
-
-let is_ipv6_inet_addr (ip : Unix.inet_addr) =
-  String.length (Obj.magic ip) = 16
-
-let is_multicast_inet_addr ip =
-  if is_ipv4_inet_addr ip then
-    logand_inet_addr
-      ip
-      (Unix.inet_addr_of_string "240.0.0.0")
-    = (Unix.inet_addr_of_string "224.0.0.0")
-  else
-    if is_ipv6_inet_addr ip then
-      logand_inet_addr
-        ip
-        (Unix.inet_addr_of_string "ffff::0")
-      = (Unix.inet_addr_of_string "ff00::0")
-    else
-      false
-
-
 external mcast_set_loop : Unix.file_descr -> bool -> unit 
   = "netsys_mcast_set_loop"
 external mcast_set_ttl : Unix.file_descr -> int -> unit 
