@@ -1,4 +1,4 @@
-(* $Id: netsys.mli 1662 2011-08-29 23:05:06Z gerd $ *)
+(* $Id: netsys.mli 2195 2015-01-01 12:23:39Z gerd $ *)
 
 (** System calls missing in the [Unix] module *)
 
@@ -25,6 +25,7 @@ type fd_style =
     | `W32_process
     | `W32_input_thread
     | `W32_output_thread
+    | `TLS of Netsys_crypto_types.file_tls_endpoint
     ]
   (** Some information what kind of operations are reasonable for descriptors:
       - [`Read_write]: The descriptor is neither a socket not one of the
@@ -60,6 +61,12 @@ type fd_style =
         Win32-specific output thread
         as returned by
         {!Netsys_win32.create_output_thread}. 
+      - [`TLS endpoint]: A TLS tunnel is running over the descriptor. The
+        details of the tunnel can be found in [endpoint]. Note that it is
+        allowed that the endpoint uses a second descriptor for either
+        reading or writing (i.e. reading and writing go via different
+        descriptors). In this case, it is sufficient that one of these
+        descriptors is accompanied with [`TLS endpoint].
 
       Win32: For the exact meaning of proxy descriptors, please see 
       {!Netsys_win32}. In short, a proxy descriptor is an abstract handle
@@ -71,7 +78,11 @@ type fd_style =
    *)
 
 val get_fd_style : Unix.file_descr -> fd_style
-  (** Get the file descriptor style *)
+  (** Get the file descriptor style.
+
+      The following styles cannot be determined automatically:
+       - [`TLS]
+   *)
 
 val gread : fd_style -> Unix.file_descr -> string -> int -> int -> int
   (** [gread fd_style fd s pos len]: Reads up to [len] bytes from 
@@ -178,6 +189,9 @@ val gshutdown : fd_style -> Unix.file_descr -> Unix.shutdown_command -> unit
          buffer contents, the underlying descriptor (not [fd]!) will be
          closed. The shutdown operation is non-blocking. If it is not
          possible at the moment of calling, the error [EAGAIN] is reported.
+       - [`TLS]: The shutdown only affects the tunnel as such, but not the
+         underlying connection. [SHUTDOWN_SEND] and [SHUTDOWN_ALL] are
+         supported. [SHUTDOWN_RECEIVE] is ignored.
        - Other styles raise [Shutdown_not_supported].
    *)
 
@@ -199,6 +213,8 @@ val is_prird : fd_style -> Unix.file_descr -> bool
       event objects. (The latter two only in the form provided by
       {!Netsys_win32}, see there.)
 
+      For [`TLS] fd styles, the functions are "best effort" only.
+
       Generally, if the blocking status cannot be determined for
       a class of I/O operations, the functions return [true], in
       the hope that it is better to block than to never conduct
@@ -224,6 +240,8 @@ val wait_until_prird : fd_style -> Unix.file_descr -> float -> bool
       event objects. (The latter two only in the form provided by
       {!Netsys_win32}, see there.)
 
+      For [`TLS] fd styles, the functions are "best effort" only.
+
       Generally, if waiting is not supported for
       a class of I/O operations, the functions return immediately [true], in
       the hope that it is better to block than to never conduct
@@ -306,6 +324,16 @@ val getpeername : Unix.file_descr -> Unix.sockaddr
 val is_absolute : string -> bool
   (** Whether this file path is absolute. Works for Unix and Win32. *)
 
+val abspath : string -> string
+  (** Return an absolute path for this file. When {!Netsys_posix.realpath}
+      is available, this function is called, and the canonical path is
+      returned. On Win32, first an arbitrary absolute path is created,
+      and then the path is tried to be simplified by resolving "." and "..".
+      If neither method works, the function raises [Invalid_argument].
+
+      Note that the file needs to exist in general.
+   *)
+
 val restart : ('a -> 'b) -> 'a -> 'b
   (** [restart f arg] calls [f arg], and restarts this call if the
     * exception [Unix_error(EINTR,_,_)] is caught.
@@ -341,6 +369,21 @@ val restarting_select :
       of descriptors than sockets.
    *)
 
+val restart_wait :
+      [ `R | `W ] -> fd_style -> Unix.file_descr -> ('a -> 'b) -> 'a -> 'b
+  (** [restart_wait mode fd_style fd f arg]: Calls [f arg], and handles
+      the following exceptions:
+       - [Unix_error(EINTR,_,_)]: Just calls [f] again
+       - [Unix_error(EAGAIN,_,_)]: waits until [fd] is readable or writable
+         as designated by [mode], and calls [f] again
+       - [Unix_error(EWOUDLBLOCK,_,_)]: same
+       - {!Netsys_types.EAGAIN_RD}: waits until [fd] is readable, and calls
+         [f] again
+       - {!Netsys_types.EAGAIN_WR}: waits until [fd] is writable, and calls
+         [f] again
+   *)
+
+
 val sleep : float -> unit
 val restarting_sleep : float -> unit
   (** Sleep for the passed time. [restarting_sleep] additionally handles
@@ -356,9 +399,11 @@ val int64_of_file_descr : Unix.file_descr -> int64
 val string_of_fd : Unix.file_descr -> string
   (** Return a string describing the descriptor (for debugging) *)
 
-val string_of_sockaddr : Unix.sockaddr -> string
+val string_of_sockaddr : ?norm:bool -> Unix.sockaddr -> string
   (** Returns a human-readable string describing the address
-      (for debug messages)
+      (for debug messages). If [norm], IPv4 addresses mapped to the IPv6
+      address space are returned in the normal dotted quad format (i.e.
+      x.y.z.u instead of ::ffff:x.y.z.u).
 
       Note that the reverse (parsing such a string) can be
       accomplished with {!Netsockaddr.socksymbol_of_string} and
@@ -382,6 +427,58 @@ external _exit : int -> unit = "netsys__exit"
    * The argument is the exit code, just as for [exit].
    *)
 
+(** {1 IPv6} *)
+
+val is_ipv6_system : unit -> bool
+  (** Whether IPv6 is available and usable. At the moment this tests for
+      the presence of a global IPv6 address on any interface. The test
+      also requires that the getifaddrs() call is available. The test
+      can be overridden with [set_ipv6_system].
+   *)
+
+val set_ipv6_system : bool -> unit
+  (** Sets whether IPv6 is usable *)
+
+
+(** {1 IP addresses} *)
+
+val logand_inet_addr : Unix.inet_addr -> Unix.inet_addr -> Unix.inet_addr
+  (** Returns the bitwise AND of the two argument addresses *)
+
+val logor_inet_addr : Unix.inet_addr -> Unix.inet_addr -> Unix.inet_addr
+  (** Returns the bitwise OR of the two argument addresses *)
+
+val logxor_inet_addr : Unix.inet_addr -> Unix.inet_addr -> Unix.inet_addr
+  (** Returns the bitwise XOR of the two argument addresses *)
+
+val lognot_inet_addr : Unix.inet_addr -> Unix.inet_addr
+  (** Returns the bitwise NOT of the argument address *)
+
+val norm_inet_addr : Unix.inet_addr -> Unix.inet_addr
+  (** Normalization: If the input address is an IPv4 address mapped into the
+      IPv6 address space, the IPv4 address is extracted. Otherwise, the input
+      address is returned unchanged.
+   *)
+
+val ipv6_inet_addr : Unix.inet_addr -> Unix.inet_addr
+  (** IPv6-ification: If the input address is for IPv4, it is mapped to the
+      IPv6 address space (so an IPv6 socket can be bound)
+   *)
+
+val is_ipv4_inet_addr : Unix.inet_addr -> bool
+  (** Whether the address is an IPv4 address (including IPv4 addresses mapped
+      into the IPv6 adress space)
+   *)
+
+val is_ipv6_inet_addr : Unix.inet_addr -> bool
+  (** Whether the address is an IPv6 address (excluding IPv4 addresses mapped
+      into the IPv6 adress space)
+   *)
+
+val is_multicast_inet_addr : Unix.inet_addr -> bool
+  (** Whether the address is a multicast address (either IPv4 or IPv6) *)
+
+
 
 (** {1 Multicast Functions} *)
 
