@@ -1,4 +1,4 @@
-/* $Id: netsys_c.c 1886 2013-08-30 11:02:36Z gerd $
+/* $Id: netsys_c.c 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  */
 
@@ -12,9 +12,9 @@ CAMLprim value netsys_int64_of_file_descr(value fd) {
 #ifdef _WIN32
     switch (Descr_kind_val(fd)) {
     case KIND_HANDLE:
-	return copy_int64((long) (Handle_val(fd)));
+	return copy_int64((intnat) (Handle_val(fd)));
     case KIND_SOCKET:
-	return copy_int64((long) (Socket_val(fd)));
+	return copy_int64((intnat) (Socket_val(fd)));
     }
     return copy_int64(0);
 #else
@@ -428,8 +428,12 @@ CAMLprim value netsys_at_fdcwd(value dummy)
 #define AT_SYMLINK_NOFOLLOW 0
 #endif
 
+#ifndef AT_SYMLINK_FOLLOW
+#define AT_SYMLINK_FOLLOW 0
+#endif
+
 static int at_flags_table[] = {
-    AT_EACCESS, AT_SYMLINK_NOFOLLOW, AT_REMOVEDIR
+    AT_EACCESS, AT_SYMLINK_NOFOLLOW, AT_SYMLINK_FOLLOW, AT_REMOVEDIR
 };
 #endif
 
@@ -479,8 +483,8 @@ CAMLprim value netsys_openat(value dirfd, value path, value flags, value perm)
     if (ret == -1) uerror("openat", path);
 #if defined(NEED_CLOEXEC_EMULATION) && defined(FD_CLOEXEC)
     if (convert_flag_list(flags, open_cloexec_table) != 0) {
-        int flags = fcntl(fd, F_GETFD, 0);
-        if (flags == -1 || fcntl(fd, F_SETFD, flags | FD_CLOEXEC) == -1)
+        int flags = fcntl(Int_val(dirfd), F_GETFD, 0);
+        if (flags == -1 || fcntl(Int_val(dirfd), F_SETFD, flags | FD_CLOEXEC) == -1)
           uerror("openat", path);
     }
 #endif
@@ -590,7 +594,7 @@ CAMLprim value netsys_symlinkat(value oldpath,
 
 CAMLprim value netsys_mkfifoat(value dirfd, value path, value mode)
 {
-#ifdef HAVE_AT
+#ifdef HAVE_MKFIFOAT
     if (mkfifoat(Int_val(dirfd), String_val(path), Int_val(mode)) == -1)
 	uerror("mkfifoat", path);
     return Val_unit;
