@@ -1,4 +1,4 @@
-/* $Id: netsys_c.h 1887 2013-08-30 12:45:48Z gerd $ */
+/* $Id: netsys_c.h 2013 2014-08-31 14:19:40Z gerd $ */
 
 #ifndef NETSYS_C_H
 #define NETSYS_C_H
@@ -162,45 +162,8 @@ extern void caml_darken (value, value *);
 #define Nothing ((value) 0)
 
 #ifdef _WIN32
-
-struct filedescr {
-  union {
-    HANDLE handle;
-    SOCKET socket;
-  } fd;
-  enum { KIND_HANDLE, KIND_SOCKET } kind;
-  int crt_fd;
-};
-
-#define Handle_val(v) (((struct filedescr *) Data_custom_val(v))->fd.handle)
-#define Socket_val(v) (((struct filedescr *) Data_custom_val(v))->fd.socket)
-#define Descr_kind_val(v) (((struct filedescr *) Data_custom_val(v))->kind)
-#define CRT_fd_val(v) (((struct filedescr *) Data_custom_val(v))->crt_fd)
-
-#define NO_CRT_FD (-1)
-
-/* These functions are actually defined in unixsupport_w32.c */
-
-value netsysw32_unix_error_of_code (int errcode);
-void netsysw32_unix_error (int errcode, char * cmdname, value arg) Noreturn;
-void netsysw32_uerror (char * cmdname, value arg) Noreturn;
-
-void netsysw32_win32_maperr(DWORD errcode);
-
-value netsysw32_win_alloc_handle_or_socket(HANDLE);
-value netsysw32_win_alloc_handle(HANDLE);
-value netsysw32_win_alloc_socket(SOCKET);
-
-/* Keep the API: */
-
-#define unix_error_of_code         netsysw32_unix_error_of_code
-#define unix_error                 netsysw32_unix_error
-#define uerror                     netsysw32_uerror
-#define win32_maperr               netsysw32_win32_maperr
-#define win_alloc_handle_or_socket netsysw32_win_alloc_handle_or_socket
-#define win_alloc_handle           netsysw32_win_alloc_handle
-#define win_alloc_socket           netsysw32_win_alloc_socket
-
+#include "caml/unixsupport.h"
+#define netsysw32_win_alloc_handle win_alloc_handle
 #else
 
 /* POSIX */
