@@ -1,4 +1,4 @@
-/* $Id: netsys_c_win32.c 1505 2010-12-15 19:15:22Z gerd $ */
+/* $Id: netsys_c_win32.c 2195 2015-01-01 12:23:39Z gerd $ */
 
 /* Note: need -lws2_32 to link this file, e.g. */
 /* ocamlmktop -o otop -I . -cclib -lws2_32 unix.cma netsys.cma */
@@ -11,7 +11,6 @@ static int debug = 0;
 
 #ifdef _WIN32
 
-#include "unixsupport_w32.c"
 #include <wincrypt.h>
 #include <stdarg.h>
 
@@ -63,6 +62,42 @@ CAMLprim value netsys_fill_random (value s) {
 }
 
 
+CAMLprim value netsys_get_full_path_name(value path) {
+#ifdef _WIN32
+    TCHAR buffer[4096];
+
+    /* FIXME: dynamic allocation of result buffer */
+    if (GetFullPathName(String_val(path),
+                        4096,
+                        buffer,
+                        NULL) == 0) {
+        win32_maperr(GetLastError());
+	uerror("GetFullPathName", Nothing);
+    };
+    return caml_copy_string(buffer);
+#else
+    invalid_argument("netsys_get_full_path_name");
+#endif
+}
+
+
+CAMLprim value netsys_get_long_path_name(value path) {
+#ifdef _WIN32
+    TCHAR buffer[4096];
+
+    /* FIXME: dynamic allocation of result buffer */
+    if (GetLongPathName(String_val(path),
+                        buffer, 
+                        4096) == 0) {
+        win32_maperr(GetLastError());
+	uerror("GetLongPathName", Nothing);
+    };
+    return caml_copy_string(buffer);
+#else
+    invalid_argument("netsys_get_long_path_name");
+#endif
+}
+
 /* The implementation in Unix is problematic because the handle is
    duplicated!
 */
