@@ -1,4 +1,4 @@
-(* $Id: netsys_posix.ml 1885 2013-08-30 10:13:41Z gerd $ *)
+(* $Id: netsys_posix.ml 2195 2015-01-01 12:23:39Z gerd $ *)
 
 open Unix
 open Printf
@@ -267,7 +267,7 @@ let create_poll_array n =
     Poll_mem(s,n)
   )
   else (
-    let e = Array.create n null_poll_cell in
+    let e = Array.make n null_poll_cell in
     Poll_emu e
   )
 
@@ -699,25 +699,26 @@ let run_post_fork_handlers() =
 
 (* "at" *)
 
-type at_flag = AT_EACCESS | AT_SYMLINK_NOFOLLOW | AT_REMOVEDIR
+type at_flag = AT_EACCESS | AT_SYMLINK_NOFOLLOW | AT_SYMLINK_FOLLOW | 
+               AT_REMOVEDIR
 
 (* The stubs assume these type definitions: *)
-IFDEF HAVE_O_CLOEXEC THEN
+#ifdef HAVE_O_CLOEXEC
 type open_flag1 = Unix.open_flag =
     O_RDONLY | O_WRONLY | O_RDWR | O_NONBLOCK | O_APPEND | O_CREAT | O_TRUNC
   | O_EXCL | O_NOCTTY | O_DSYNC | O_SYNC | O_RSYNC | O_SHARE_DELETE
   | O_CLOEXEC
-ELSE
-IFDEF HAVE_O_SHARE_DELETE THEN
+#else
+#ifdef HAVE_O_SHARE_DELETE
 type open_flag1 = Unix.open_flag =
     O_RDONLY | O_WRONLY | O_RDWR | O_NONBLOCK | O_APPEND | O_CREAT | O_TRUNC
   | O_EXCL | O_NOCTTY | O_DSYNC | O_SYNC | O_RSYNC | O_SHARE_DELETE
-ELSE
+#else
 type open_flag1 = Unix.open_flag =
     O_RDONLY | O_WRONLY | O_RDWR | O_NONBLOCK | O_APPEND | O_CREAT | O_TRUNC
   | O_EXCL | O_NOCTTY | O_DSYNC | O_SYNC | O_RSYNC
-ENDIF
-ENDIF
+#endif
+#endif
 
 type access_permission1 = Unix.access_permission =
     R_OK | W_OK | X_OK | F_OK
