@@ -1,4 +1,4 @@
-(* $Id: netsys_sem.ml 1776 2012-04-18 16:03:18Z gerd $ *)
+(* $Id: netsys_sem.ml 1879 2013-08-19 09:46:14Z gerd $ *)
 
 module Debug = struct
   let enable = ref false
@@ -31,7 +31,9 @@ module Emu = struct
        the code.
      *)
 
-  let n_active = Netsys_posix.sysconf_open_max() / 4
+  let n_active =
+    try Netsys_posix.sysconf_open_max() / 4
+    with _ -> 64
     (* max number of open semaphores (normally, each open sem consumes
        a file descr). It is tried not to exceed this value, but in some
        situations this will nevertheless occur.
@@ -182,6 +184,7 @@ module Emu = struct
 	  Netsys_posix.SHM_O_CREAT
 	]
 	0o600 in
+    let fd_open = ref true in
     ( try
 	let st = Unix.fstat fd in
 	if st.Unix.st_size = 0 then
@@ -189,6 +192,7 @@ module Emu = struct
 	let used = 
 	  Netsys_mem.memory_map_file fd true n in
 	Unix.close fd;
+        fd_open := false;
         let mutex =
           Netsys_posix.sem_open
 	    (prefix ^ "_contsem")
@@ -207,7 +211,8 @@ module Emu = struct
           id = Oo.id (object end);
 	}
       with error ->
-	Unix.close fd;
+        if !fd_open then 
+	  Unix.close fd;
 	raise error
     )
 
@@ -697,5 +702,9 @@ let sem_value_max =
   Netsys_posix.sem_value_max
 
 let sem_size =
-  max 4 Netsys_posix.sem_size
+  (* should be a multiple of the word size, so we round it up to the
+     next multiple of 8
+   *)
+  let s = max 4 Netsys_posix.sem_size in
+  (((s-1) / 8)+1) * 8
 
