@@ -1,4 +1,4 @@
-(* $Id: netsys_win32.ml 1505 2010-12-15 19:15:22Z gerd $ *)
+(* $Id: netsys_win32.ml 2195 2015-01-01 12:23:39Z gerd $ *)
 
 (* Security of named pipes:
 
@@ -10,6 +10,8 @@ open Printf
 
 external fill_random : string -> unit = "netsys_fill_random"
 
+external get_full_path_name : string -> string = "netsys_get_full_path_name"
+external get_long_path_name : string -> string = "netsys_get_long_path_name"
 
 type c_event
 type c_pipe_helper
