@@ -1,4 +1,4 @@
-(* $Id: rpc_auth_local.ml 1671 2011-09-22 22:25:06Z gerd $
+(* $Id: rpc_auth_local.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -49,7 +49,8 @@ object
       )
 
   method authenticate _ _ _ _ = ()
-
+  method invalidate() = ()
+  method invalidate_connection _ = ()
 end
 
 
