@@ -1,8 +1,9 @@
-(* $Id: syntax.ml 1701 2012-02-14 14:48:46Z gerd $
+(* $Id: syntax.ml 1871 2013-07-30 22:13:52Z gerd $
  * ----------------------------------------------------------------------
  *)
 
 open Netnumber;;
+open Printf
 
 exception Error of string;;
 
@@ -15,6 +16,14 @@ type xdr_id =
     }
 ;;
 
+type mangling_option =
+  [ `Lowercase
+  | `Uppercase
+  | `Capitalize
+  | `Prefix of string
+  ]
+;;
+
 let mk_id name =
   { xdr_name = name; ocaml_name_requested = false; ocaml_name = "" }
 ;;
@@ -27,6 +36,48 @@ let mk_void_id () =
   { xdr_name = ""; ocaml_name_requested = false; ocaml_name = "" }
 ;;
 
+let mangle_name opts name =
+  List.fold_left
+    (fun name opt ->
+       match opt with
+         | `Lowercase -> String.lowercase name
+         | `Uppercase -> String.uppercase name
+         | `Capitalize -> String.capitalize name
+         | `Prefix p -> p ^ name
+    )
+    name
+    opts
+;;
+
+let mangle_id ?(remap=false) opts id =
+  (* remap: only true for the second round of mangling of union cases.
+     As we want to map the cases independently of the base enum, we ignore
+     the ocaml_name for the base enum.
+   *)
+  if remap then (
+    if opts <> [] then
+      { id with
+          ocaml_name = mangle_name opts id.xdr_name;
+          ocaml_name_requested = true;
+      }
+    else
+      { id with
+        ocaml_name = String.lowercase id.xdr_name;
+        ocaml_name_requested = false;
+      }
+  )
+  else (
+    if opts <> [] && not id.ocaml_name_requested then
+      { id with
+          ocaml_name = mangle_name opts id.xdr_name;
+          ocaml_name_requested = true;
+      }
+    else
+      id
+  )
+;;
+
+
 type xdr_constant =
   | Constant of (bool * uint4)        (* sign (true=neg), absolute value *)
   | Named_constant of string
@@ -45,6 +96,12 @@ type int_variant =
   | Unboxed
 ;;
 
+type struct_option =
+  [ `Tuple 
+  | `Equals of string
+  ]
+;;
+
 type xdr_type =
     T_opaque_fixed of xdr_constant ref
   | T_opaque of xdr_constant ref
@@ -67,7 +124,7 @@ type xdr_type =
   | T_bool
   | T_refer_to of (refconstr * string ref)         (* always the XDR name! *)
   | T_enum of (xdr_id * xdr_constant ref) list
-  | T_struct of xdr_decl list
+  | T_struct of (struct_option list * xdr_decl list)
   | T_union of xdr_union
 
 and refconstr =
@@ -85,6 +142,10 @@ and xdr_union =
 	 * ocaml_name: The ocaml name for the polymorphic variant
 	 * component_decl: The declaration after ':' (t x)
 	 *)
+      mangling : mangling_option list;
+        (* Additional mangling for [cases]. This is delayed because
+           [discrimant] can be a reference.
+         *)
       default : xdr_decl option;
     }
 
@@ -123,6 +184,14 @@ type xdr_def =
 ;;
 
 
+let mk_enum ?remap (opts:mangling_option list) l =
+  List.map
+    (fun (id,const) ->
+       (mangle_id ?remap opts id,const)
+    )
+    l
+
+
 let mk_decl id t =
   { decl_type = t;
     decl_symbol = id;
@@ -130,13 +199,40 @@ let mk_decl id t =
   }
 ;;
 
-let mk_union discr cs defl =
+let mk_union opts discr cs defl =
   { discriminant = discr;
     cases = cs;
+    mangling = opts;
     default = defl;
   }
 ;;
 
+let mangling_opt opt =
+  match opt with
+    | #mangling_option as opt -> [opt]
+    | _ -> []
+
+let struct_opt opt =
+  match opt with
+    | #struct_option as opt -> [opt]
+    | _ -> []
+
+
+let mk_struct opts decls =
+  let mangling_opts =
+    List.flatten (List.map mangling_opt opts) in
+  let struct_opts =
+    List.flatten (List.map struct_opt opts) in
+  let decls1 =
+    List.map
+      (fun d ->
+         { d with decl_symbol = mangle_id mangling_opts d.decl_symbol }
+      )
+      decls in
+  (struct_opts, decls1)
+;;
+
+
 let mk_program symbol def number =
   { prog_symbol = symbol;
     prog_def = def;
@@ -262,7 +358,7 @@ let resolve_constants dl =
 				    resolve_const c;
 				    def_constant id (constant !c);
 				 ) l
-      | T_struct td         -> List.iter check_type_decl td
+      | T_struct(_,td)      -> List.iter check_type_decl td
       | T_union u           -> check_type_decl (u.discriminant);
 	                       List.iter (fun (c,_,td) ->
 					    resolve_const c;
@@ -427,7 +523,7 @@ let check_type_constraints dl =
       | T_refer_to n         -> ignore(get_type t)
       | T_enum l             -> List.iter
 	                          (fun(_,c) -> signed_int (constant !c)) l
-      | T_struct td          -> uniqueness_in_tdlist td;
+      | T_struct(_,td)       -> uniqueness_in_tdlist td;
 	                        List.iter check_type_decl td
       | T_union u            -> let defdecl =
 	                          match u.default with
