@@ -1,4 +1,4 @@
-(* $Id: rpc_xti_client.mli 1003 2006-09-24 15:17:15Z gerd $
+(* $Id: rpc_xti_client.mli 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -18,16 +18,3 @@ val cots_connect : string -> string -> Unix.file_descr
    * The "tirdwr" module is pushed onto the STREAM such that
    * the "read" and "write" syscalls work.
    *)
-
-(** {2 Contact the keyserv daemon over XTI} *)
-
-type connector =
-    [ `Direct of (Rpc_client.connector * Rpc.protocol)
-    | `Keyenvoy of string
-    ]
-  (** Same as {!Rpc_key_service.connector} *)
-
-val keyserv_connector : connector
-  (** Returns a connector that can be used to call the
-   * keyserv daemon.
-   *)
