@@ -1,11 +1,8 @@
-(* $Id: rpc_common.ml 258 2004-05-25 16:49:11Z gerd $
+(* $Id: rpc_common.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
 
-open Rtypes
-open Xdr
-
 (* auxiliary functions of general use *)
 
 let rec unix_call (f:unit->'a) : 'a =
@@ -14,8 +11,3 @@ let rec unix_call (f:unit->'a) : 'a =
   with
     Unix.Unix_error (Unix.EINTR, _, _) -> unix_call f
 ;;
-
-
-(* TODO:
- * check if all system calls use unix_call
- *)
