@@ -1,4 +1,4 @@
-(* $Id: rpc_program.mli 1251 2009-05-29 01:57:46Z gerd $
+(* $Id: rpc_program.mli 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -20,8 +20,8 @@
  * (a set of definitions of type names).
  *)
 
-open Rtypes
-open Xdr
+open Netnumber
+open Netxdr
 
 type t
   (** Type of RPC programs *)
