@@ -1,4 +1,4 @@
-(* $Id: rpc_simple_client.ml 258 2004-05-25 16:49:11Z gerd $
+(* $Id: rpc_simple_client.ml 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -7,7 +7,7 @@
  * in a relatively simple way.
  *)
 
-open Xdr;;
+open Netxdr;;
 open Rpc;;
 
 type t =
