@@ -1,4 +1,4 @@
-(* $Id: rpc_simple_client.mli 279 2006-04-24 14:45:15Z gerd $
+(* $Id: rpc_simple_client.mli 2195 2015-01-01 12:23:39Z gerd $
  * ----------------------------------------------------------------------
  *
  *)
@@ -9,7 +9,7 @@
  * of RPC.
  *)
 
-open Xdr
+open Netxdr
 
 type t
   (** The type of simple clients *)
